{ =====================================================
 Copyright (C) 2007 Francisco A. Ocana & Pablo Onate 

 This file is part of IndElec

 IndElec is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 IndElec is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You have received a copy of the GNU General Public License
 in a file called COPYING which you should have received with
 IndElec. If not, see <http://www.gnu.org/licenses/>.
 ===================================================== 
}
unit ADIMINOU;

{ 
Routines to create output files of the module Dimensi 
with aggregated data.

Unidad que contiene las rutinas input - ouput de datos del
programa Dimensi}

{$N+}


INTERFACE

uses COMUN, DIMDEFN;

procedure leeParty(var f: text; var rPa: Tparty);

procedure iniDato(var infoFi: TOInfoFich; var f: text;
                  var noParti, nodat: TRgPartido);

procedure iniResultado(var infoFi: TOInfoFich; var f: text;
                       noParti: TRgPartido);
procedure iniResultHTML(var infoFi: TOInfoFich; var f: text;
                        noParti: TRgPartido);

procedure iniResultRS(var infoFi: TOInfoFich; var f: text;
                       noParti: TRgPartido);

procedure writeDatoTXT(var f: text; var Pa: PVecParty;
                       noParti, nodat: TRgPartido;
                       var totv, tote: Tnumero);
procedure writeDatoHTML(var f: text; var Pa: PVecParty;
                       noParti, nodat: TRgPartido;
                       var totv, tote: Tnumero);



procedure TheEnd(var Pa: PVecParty;
                 noParti: TRgPartido);




IMPLEMENTATION

uses COMUTIL, UHTML, MEMBRETE;


procedure leeParty(var f: text; var rPa: Tparty);
{
This procedure reads a party record (acronym, votes and seats) 
from the text data file "f", which is returned to "rPa".


      Este procedimiento lee, del fichero input asignado en "f" (global),
los datos de un partido poltico: nombre, votos y escaos.  
Los contenidos de votos y escaos son almacenados en rPa, .

}

  var
      hechoCadena,
      iniCadena: boolean;
      cont: byte;
      c: char;

Begin
  rPa.SiglasCandi:= '';

  hechoCadena:= FALSE;
  iniCadena:= FALSE;

  cont:= 0;

  repeat
    read(f,c);

    if c = ' '
    then
      begin
        if iniCadena then hechoCadena:= TRUE;
      end
    else
      begin
        iniCadena:= TRUE;
        if cont < lenCandida
        then rPa.SiglasCandi:= rPa.SiglasCandi + c;

        Inc(cont);
      end;

  until hechoCadena;

  readln(f, rPa.Vp, rPa.Ep);
End;


procedure iniDato(var infoFi: TOInfoFich; var f: text;
                  var noParti, nodat: TRgPartido);

{
 This procedure carries out the data loading process.
 In particular, it performs the following actions:
 - specification of the path and name of the *.dat input file;
 - creation of the file DATOS.aux of Tparty records associated
   to each polical party.

  Este procedimiento lleva a cabo el proceso de introduccin de datos.
  En concreto, realiza las siguientes funciones:
   - Especificacin del camino y nombre del fichero input (.dat);
   - Creacin del fichero DATOS.aux con registros Tparty para cada
     partido poltico.
 }


 var     
   g: file of Tparty;
   rpa: Tparty;


Begin
  assign(f, infoFi.camin + infoFi.nom + '.dat');  reset(f);

  leeTituloFich(infoFi.camin, f);
  readln(f);    { Skip second line. Its content is not read }

  nodat:= 0;
  noParti:= 0;

  assign(g, camIndElec+'DATOS.aux');   rewrite(g);

  repeat
    leeParty(f, rpa);
    write(g, rpa);

    if rpa.Vp > 0 then Inc(nodat);

    Inc(noParti);
  until eof(f);

  close(g);
  close(f);

  Dec(noParti);
  Dec(nodat);
End;
{ *********************************************************** }

procedure iniResultado(var infoFi: TOInfoFich; var f: text;
                       noParti: TRgPartido);

{
  Inicializa el fichero "f" para escribir en l, e introduce
     - Ttulo
     - Nmero total de partidos polticos
}


Begin
  assign(f, infoFi.camin + infoFi.nom+'.out');   rewrite(f);

  MmbrIndx(f, 0, 't');

  showTitulo(infoFi.camin, f, 't');

  writeln(f);
  writeln(f,' Number of parties read from data file = ', Succ(noParti));
  writeln(f);

End;

{ *********************************************************** }

procedure iniResultHTML(var infoFi: TOInfoFich; var f: text;
                        noParti: TRgPartido);

{
  Inicializa el fichero "f" para escribir en l, e introduce
     - Ttulo
     - Nmero total de partidos polticos
  Versin HTML de  iniResultado.
}


Begin
  assign(f, infoFi.camin + infoFi.nom+'.htm');   rewrite(f);
  writeln(f, '<html>');
  writeln(f, '<head>');
  IntroMetaHTML(f);
  writeln(f, '<title>');
  showTitulo(infoFi.camin, f, 'h');
  writeln(f, '</title> </head>');
  writeln(f, '<body>');

  MmbrIndx(f, 0, 'h');

  write(f, '<h2>');
  showTitulo(infoFi.camin, f, 'h');
  write(f, '</h2>');

  writeln(f, '<p>');
  writeln(f,'<b>Number of parties read from data file =</b> ', Succ(noParti));
  writeln(f, '</p>');

End;
{ ************************************************************** }


procedure iniResultRS(var infoFi: TOInfoFich; var f: text;
                       noParti: TRgPartido);

{
  Version of iniResultado for the R/S language  
}


Begin
  assign(f, infoFi.camin + infoFi.nom+'.R');   rewrite(f);

  MmbrIndx(f, 0, 'r');

  showTitulo(infoFi.camin, f, 'r');
  writeln(f);

  write(f, infoFi.nom);
  write(f, '<- list(');
  write(f, 'Number.read.parties = ', Succ(noParti));
  writeln(f, ')');
  writeln(f);
End;



procedure writeDatoTXT(var f: text; var Pa: PVecParty;
                       noParti, nodat: TRgPartido;
                       var totv, tote: Tnumero);

  const tabcolu = 10;
        tabAcu = 14;

  var i: TRgPartido;
      g: file of Tparty;
      rPa: Tparty;


Begin
  writeln(f);
  writeln(f, 'DATA AND DISTRIBUTIONS OF VOTES AND SEATS':60);
  writeln(f);

  assign(g, camIndElec+'DATOS.aux');    reset(g);

  write(f, 'No':4);                write(f, charTabu);
  write(f, 'Party':lenCandida);  write(f, charTabu);
  write(f,'Votes':tabcolu);        write(f, charTabu);
  write(f,'Seats':tabcolu);      write(f, charTabu);
  write(f,'%Votes':tabcolu);      write(f, charTabu);
  write(f,'%Seats':tabcolu);    writeln(f, charTabu);

  totv:= 0; tote:=0;

  for i:= 0 to noParti do
    begin
      read(g, rPa);
      
      write(f, Succ(i):4);   write(f, charTabu);

      write(f, rPa.SiglasCandi: lenCandida);   write(f, charTabu);

      escridatEE(f, rPa.Vp, tabcolu, preci, TRUE, FALSE);
      totv:= totv + rPa.Vp;

      escridatEE(f, rPa.Ep, tabcolu, preci, TRUE, FALSE);
      tote:= tote + rPa.Ep;

      escridat(f, Pa^[i].Vp, tabcolu, preci, TRUE, FALSE);
      escridat(f, Pa^[i].Ep, tabcolu, preci, TRUE, TRUE);
    end;

  close(g);

  writeln(f);
  writeln(f,' Sum of votes = ', totv:2:2);
  writeln(f,' Sum of seats = ', tote:2:2);
  writeln(f);

  writeln(f);
  writeln(f);
  writeln(f, 'CUMULATIVE DISTRIBUTIONS OF VOTES AND SEATS':54);
  writeln(f);

  write(f,'No':4);                   write(f, charTabu);
  write(f, '%Cum. Votes':tabAcu);    write(f, charTabu);
  write(f, '%Cum. Seats':tabAcu);    writeln(f, charTabu);

  rPa.Vp:= 0;
  rPa.Ep:= 0;

  for i:= 0 to noParti do
    begin
      rPa.Vp:= rPa.Vp + Pa^[i].Vp;
      rPa.Ep:= rPa.Ep + Pa^[i].Ep;
      
      write(f, Succ(i):4);  write(f, charTabu);
      escridat(f, rPa.Vp, tabAcu, preci, TRUE, FALSE);
      escridat(f, rPa.Ep, tabAcu, preci, TRUE, TRUE);
    end;

  writeln(f);
  writeln(f,' Number of parties with electoral support = ', Succ(nodat));

  writeln(f);
  writeln(f);
End;

{ ******************************************************************* }

procedure writeDatoHTML(var f: text; var Pa: PVecParty;
                       noParti, nodat: TRgPartido;
                       var totv, tote: Tnumero);

{ versin HTML de escribeDatos }

  var i: TRgPartido;
      g: file of Tparty;
      rPa: Tparty;


Begin
  writeln(f, '<h2>');
  write(f, 'Data and distributions of votes and seats');
  writeln(f, '</h2>');

  assign(g, camIndElec+'DATOS.aux');      reset(g);

  writeln(f, '<p><table cellspacing="5"> <tr>');

  writeHeadTabla(f, 'No', 'r');
  writeHeadTabla(f, 'Party', 'l');
  writeHeadTabla(f,'Votes', 'r');
  writeHeadTabla(f,'Seats', 'r');
  writeHeadTabla(f,'%Votes', 'r');
  writeHeadTabla(f,'%Seats', 'r');
  writeln(f, '</tr>');

  totv:= 0; tote:= 0;

  for i:= 0 to noParti do
    begin
      read(g, rPa);

      writeln(f,'<tr>');
      write(f, '<td align="right">');  write(f,  Succ(i));
      writeln(f, '</td>');

      writeItmTabla(f, rPa.SiglasCandi, 'l');

      escridaEEHT(f, rPa.Vp, preci);
      totv:= totv + rPa.Vp;

      escridaEEHT(f, rPa.Ep, preci);
      tote:= tote + rPa.Ep;

      escridaHT(f, Pa^[i].Vp, preci);
      escridaHT(f, Pa^[i].Ep, preci);
      writeln(f, '</tr>');
    end;

  close(g);
  writeln(f, '</table> </p>');

  writeln(f, '<p>');
  writeln(f,'<b>Sum of votes =</b> ', totv:2:2);
  writeln(f, '<br>');
  writeln(f,'<b>Sum of seats =</b> ', tote:2:2);
  writeln(f, '</p> <hr> <p> </p>');

  writeln(f, '<h2>');
  write(f, 'Cumulative distributions of votes and seats');
  writeln(f, '</h2>');

  writeln(f, '<p><table cellspacing="5"> <tr>');
  writeHeadTabla(f,'No', 'r');
  writeHeadTabla(f, '%Cum. Votes', 'r');
  writeHeadTabla(f, '%Cum. Seats', 'r');
  writeln(f, '</tr>');

  rPa.Vp:= 0;
  rPa.Ep:= 0;

  for i:= 0 to noParti do
    begin
      rPa.Vp:= rPa.Vp + Pa^[i].Vp;
      rPa.Ep:= rPa.Ep + Pa^[i].Ep;

      writeln(f, '<tr>');
      write(f, '<td align="right">');
      write(f, Succ(i));
      writeln(f, '</td>');

      escridaHT(f, rPa.Vp, preci);
      escridaHT(f, rPa.Ep, preci);
      writeln(f, '</tr>');
    end;
  writeln(f, '</table> </p>');

  writeln(f, '<p>');
  writeln(f,'<b>Number of parties with electoral support =</b> ', Succ(nodat));
  writeln(f, '</p> <hr> <p> </p>');


End;



{ ******************************************************************* }

procedure TheEnd(var Pa: PVecParty; noParti: TRgPartido);

Begin
  FreeMem(Pa, TlongNatural(Succ(noParti)) * sizeOf(Tparty) );
End;




END.
