{ =====================================================
 Copyright (C) 2007 Francisco A. Ocana & Pablo Onate 

 This file is part of IndElec

 IndElec is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 IndElec is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You have received a copy of the GNU General Public License
 in a file called COPYING which you should have received with
 IndElec. If not, see <http://www.gnu.org/licenses/>.
 ===================================================== 
}
unit COMUN;

{$N+}

{
This unit contains the definitions of global constants and global types of data
for IndElec. Among such definitions, the constants setting the IndElec output 
formats are found in this unit.
}

INTERFACE



const

 NoMaxParti = 220;  {Maximum number of parties in a district.  
                    }
 lenCandida = 15;  {Maximum length of a party acronym string.
                 Longitud de cadena para siglas de partidos 
                 polticos (Candidatura). 
                 }  
 lenItmDigits = 2; {Maximum number of digits for the codes of items  
                 for an aggregation level.
                   }
 preci = 3; { Precision for real numbers in output text files. 
           Precisin : convertir al final como variable global
           a especificar en fichero de configuracin, por ejemplo.  
            }    
 camIndElec = '\indelec\bin\';              
             { IndElec setup path
             * Camino de trabajo auxiliar por defecto de IndElec. 
              '\indelec\bin\';   '\indelec\delfi1\';
              \indelec\lazaro\
             }

{ Module Dimensi:
  ==============

  Constants indicating the positions of indices in the binary files (*.D?)
  constructed in Dimensi. The purpose of these constants consists
  of making the adding of new indexes easy.
 
  Constantes asociadas a los ndices correspondientes a cada uno 
  los ndices electorales.   
}


 iiDspro = 1;  	{starting position: disproportionality}

 nnDspro = 11;  {total number of indices of disproportionality}

 iiDsRobu = iiDspro+nnDspro; {starting position: robust indices of disproportionality }
     {12=ndice de inicio de desproporcionalidad robusta }

 nnDsRobu = 4;  {total number of robust indices of disproportionality }


 iiSesg = iiDsRobu+nnDsRobu; {starting position: indices of bias}
  { 16=ndice de inicio de ndices de sesgo }
 
 nnSesgLS = 3;  {number of indices of bias based on least squares except r2 }
  {no. nd. de sesgo basados en regresin LS, sin contar los r2 }
 nnSesgTuk = 4; {number of indices of bias based on Tukey line except r2 }
  {no. nd. de sesgo basados en lnea Tukey, sin contar los r2 }
 
 nnSesg = nnSesgLS+nnSesgTuk; {total number of indices of bias except r2} 
  { 7 = no. total de tipos de ndices de sesgo sin contar los coeficientes r2 } 
 nnSesgTT = nnSesg*2;   {total numbers of indices of bias and r2}
  { 14 = no. total indices de sesgo y R2's }
       

 iiFrgCCcm = iiSesg + nnSesgTT; {starting position: Frag, CC, Cmp indices}
  {30=ndice de inicio de ndices de Frag, CC, Cmp }
 
 nnFrgCCcm = 6; {total number: Frag, CC, Cmp indices}
  { no. total de ndices Frag, CC, Cmp }


 iiNoEff = iiFrgCCcm + nnFrgCCcm; {starting position: indices of effective number of parties}
  {36= ndice de inicio de ndices del no. efectivo de partidos}

 nnNoEff = 9; {number: indices of effective number of parties}
   { no. total de valores de ndices de no. efectivo de partidos. }


 iiPolar = iiNoEff + nnNoEff; {starting position: indices of polarization}
  {45=ndice de inicio de ndices de polarizacin}

 nnPolar = 4; {total number: indices of polarization}
  { no. total de valores de ndices de polarizacin }


 NNIndexs = nnDspro + nnDsRobu + 2*nnSesg + nnFrgCCcm + nnNoEff + nnPolar;   
   { total number of indices of disproportionality and party dimensions
     48 = no. total de ndices de desproporcionalidad y 
            de dimensiones de partidos }


{ ================================
 Constants setting the output files
 ======================================
 Constantes asociadas a la presentacin 
 de resultados
 ======================================
}

 MaxPartiPrint = 9;  { maximum number of most voted parties to be shown |
                no. mximo de partidos impresos, cuando se construyen los ficheros
                salidas de matrices de datos del mdulo Dimensi con datos agregados }

 charTabu = #9;  { tabulator char }

 sepacsv = ','; { separator for the CSV format file }

 cdistri = 0;  
{     	This parameter especifies the distribution considered in the
	statistical report elaborated by IndElec. 
    0  uniform distribution
	  1  distribution of votes
	  2  distribution of seats	

      parmetro que especifica la distribucin considerada a la hora de
      obtener la estadstica descriptiva de las medidas electorales obtenidas,
      que aperece en los informes elaborados por IndElec, con la excepcin de
      los ndices de party linkage
              0  distribucin uniforme;          1  distribucin de voto;
              2  distribucion de escaos
}


{ 
  Constants for the electoral regionalism
  =======================================

  Constants indicating the positions of indices in the binary files (*.reg)
  constructed in Dimensi for regionalism. The purpose of these constants consists
  of making easy the adding of new indexes.

  Constantes asociadas a los ndices correspondientes a cada uno 
  los ndices electorales (regionalismo y de party linkage)
  almacenados en los ficheros del regionalismo *.reg 

}

 nnRegi0Fi = 8;  
{  Number of initial records of the *.reg and *.aux files.
   Nmero de registros al inicio 
   del fichero *.reg (idem para el fichero *.aux) ,
   sin contar los EqNo+1 iniciales }

 iiRegi = 1;   {starting point: indices of regionalism}
   { posicin de inicio del regionalismo}
 nnRegi = 4;   {total number of indices of regionalism}
   { no. total de ndices del regionalismo }

 iiENP = iiRegi+nnRegi; {starting point: indices of effective number of parties (ENP) }
    {ndice de inicio de los ndices ENP }  
 nnENP = 4; { number of indices of ENP }
    {nmero de ndices ENP } 
 
 NNindxFichReg = nnRegi + nnENP; {total number of computed indices in the *.reg files.}  
    { no. total de ndices de los ficheros *.reg }


{ T y p e s   o f   d a t a   }

type
  TnombreFi = string[8];    { file names  }
  TextenFi = string[3];     { extension of files, without point. }
  TcaminFi = string[60];    { working path }


  TcadenaTi = string[85];   { short title describing data- only in a line }
  Tcadenhtml = string[50];  { HTML items }
  Tcadanchor = string[10]; { HTML anchor }


  TcadCandi = string[lenCandida]; { string of political party acronyms-names
				Nombre de candidaturas o siglas de partidos polticos }

  Tcadena1 = string[1];  {auxiliary string of length 1}
  Tcadreal = string[6];                              

  TlongNatural = Longword; {LongInt}
{   The widest range for natural numbers. For instance, it is the range for the
    record positions in a file. }

  Tnumero0 = real;       { real numbers in ouput routines: the least accurate real numbers }
  Tnumero = real;        { real numbers }
  Tnumero2 = extended;   { the most accurate real numbers }

  TRgPartido = byte;    { range of political parties = 0...NomaxParti }

  Tfichero = file of Tnumero;  { binary files of real numbers}

  TOInfoFich =  object
    camin: TcaminFi;
    nom: TnombreFi;
    procedure inicializa(caminx: TcaminFi);
  end;

  POInfoFich = ^TOInfoFich;



IMPLEMENTATION



procedure TOInfoFich.inicializa(caminx: TcaminFi);
{
From a complete file path (path+file name and extension), 
it extracts the path and the name in "caminx", and then it
assigns them to "camin" and "nom", respectively.
}
  var l: byte;
Begin
  camin:= caminx;

  l:= length(camin);

  l:= l-4;
  nom := '';

  repeat
    nom:= camin[l] + nom;
    Dec(l);
  until (l=0) or (camin[l] = '\');

  camin[0] := chr(l);
End;



{ FIN DE COMUN }

END.
