{ =====================================================
 Copyright (C) 2007 Francisco A. Ocana & Pablo Onate 

 This file is part of IndElec

 IndElec is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 IndElec is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You have received a copy of the GNU General Public License
 in a file called COPYING which you should have received with
 IndElec. If not, see <http://www.gnu.org/licenses/>.
 ===================================================== 
}
unit COMUTIL;

{ 
Auxiliary Routines for the output files. They write integer and
real values into ouput files, for several format files.

Rutinas comunes utilizadas con frecuencia  
}

INTERFACE

uses COMUN;


{ Writing in text files 
  ===================
}
procedure escridat(var ft: text;  r: Tnumero0;
                   tabul, precis: byte;
                   tabulando, salto:boolean);

procedure escridatEE(var ft: text;  r: Tnumero0;
                   tabul, precis: byte;
                   tabulando, salto:boolean);


{ Writing in CSV files
  ====================
}

procedure introComaCSV(var ft: text; veces: byte);

procedure escridatCSV(var ft: text;  r: Tnumero0; salto: boolean);
procedure escridatEEcsv(var ft: text;  r: Tnumero0; salto: boolean);



{ Writing in HTML files
  =====================
}
procedure escridaHT(var ft: text; r: Tnumero0; precis: byte);
procedure escridaEEHT(var ft: text; r: Tnumero0; precis: byte);

{ Writing in list-like files
  =====================
}

procedure escridatlist(var ft: text;  r: Tnumero0; precis: byte;
                       salto, final: boolean);
procedure escridatEElist(var ft: text; r: Tnumero0; precis: byte;
                         salto, final: boolean);



{ Miscelanea }

function CheckPath(var cami: TcaminFi): boolean;

procedure leeTituloFich(var cami: TcaminFi; var f: text);

procedure showTitulo(var cami: TcaminFi; var f: text; tipfich: char);


IMPLEMENTATION

const precisix = 10;  { internal precision for real numbers in CSV files }


type
   TcadNumero = string[40];  { string storing real numbers }



procedure escridat(var ft: text; r: Tnumero0;
                   tabul, precis: byte;
                   tabulando, salto: boolean);

{
 Given a real number "r", this procedure writes it in 
 the (open) text file "ft" acording to the following settings:
   "tabul": number of tabulator spaces;
   "precis": number of decimal digits;
   "tabulando": enter tabulator char?
   "salto": enter new line char?

 Procedimiento para nmeros que puedan ser indistintamente reales
 o enteros. Si son enteros, le coloca .000
  Dado
       ft = fichero de texto dispuesto para escribir en l,
       r  = nmero,
    tabul = tabulador (no. de caracteres),
   precis = nmero de cifras decimales (precisin),
tabulando = introduzco caracter tabulador despues de escribir el valor?
    salto = salto a la lnea siguiente despues de escribir ?

 este procedimiento escribe el valor "r" en el fichero "ft" con slo
 las cifras decimales que aparecen en "precis", utilizando un
 nmero de columnas igual a "tabul".

 }

Begin
  if salto
  then
      writeln(ft, r:tabul:precis)
  else
    begin
      write(ft, r:tabul:precis);
      if tabulando then write(ft, charTabu);
    end;
End;
{ ************************************************ }

procedure escridatEE(var ft: text;  r: Tnumero0;
                   tabul, precis: byte;
                   tabulando, salto:boolean);

{  Given a real number "r", this procedure writes it in 
 the (open) text file "ft" acording to the following settings:
   "tabul": number of tabulator spaces;
   "precis": number of decimal digits;
   "tabulando": enter tabulator char?
   "salto": enter new line char?
 It also detects the integer numbers being written without decimal digits.

 Procedimiento para nmeros que puedan ser enteros.

  Dado
       ft = fichero de texto dispuesto para escribir en l,
       r  = nmero,
    tabul = tabulador (no. de caracteres),
   precis = nmero de cifras decimales (precisin),
tabulando = introduzco caracter tabulador despues de escribir el valor?
    salto = salto a la lnea siguiente despues de escribir ?

 este procedimiento escribe el valor "r" en el fichero "ft" con slo
 las cifras decimales que aparecen en "precis", utilizando un
 nmero de columnas igual a "tabul".


 }

  var cade: TcadNumero;

Begin
  if ABS( FRAC(r) ) > 0
  then
      str(r:0:precis, cade)
  else
      str(r:0:0, cade);

  if salto
  then
      writeln(ft, cade:tabul)
  else
    begin
      write(ft, cade:tabul);
      if tabulando then write(ft, charTabu);
    end;
End;
{ ************************************************ }


procedure introComaCSV(var ft: text; veces: byte);
var ix: byte;
Begin
  for ix:= 1 to veces do write(ft, sepacsv);
End;


procedure escridatCSV(var ft: text;  r: Tnumero0;
                      salto: boolean);

{  Given a real number "r", this procedure writes it in 
 the (open) CSV file "ft" acording to the following settings:
   "tabul": number of tabulator spaces;
   "precis": number of decimal digits;
   "tabulando": enter tabulator char?
   "salto": enter new line char?


 Procedimiento para nmeros que puedan ser indistintamente reales
 o enteros. Si son enteros, le coloca .000
  Dado
       ft = fichero de texto dispuesto para escribir en l,
       r  = nmero,
  tabulando = introduzco caracter tabulador despues de escribir el valor?
    salto = salto a la lnea siguiente despues de escribir ?

 este procedimiento escribe el valor "r" en el fichero "ft" con 
 las cifras decimales que aparecen en "precisix". La precisin podr ser
 modificada con posterioridad a travs de Excel. Por lo que interesa
 que precisix sea grande.

 }



Begin
  write(ft, sepacsv);
  write(ft, r:0:precisix);

  if salto then writeln(ft);
End;
{ ************************************************ }


procedure escridatEEcsv(var ft: text; r: Tnumero0; salto: boolean);

{  Given a real number "r", this procedure writes it in 
 the (open) csv file "ft" acording to the following settings:
   "tabul": number of tabulator spaces;
   "precis": number of decimal digits;
   "tabulando": enter tabulator char?
   "salto": enter new line char?
 It also detects the integer numbers being written without decimal digits.

 Procedimiento para nmeros que puedan ser enteros.

  Dado
       ft = fichero de texto dispuesto para escribir en l,
       r  = nmero,
tabulando = introduzco caracter tabulador despues de escribir el valor?
    salto = salto a la lnea siguiente despues de escribir ?

 este procedimiento escribe el valor "r" en el fichero "ft" con slo
 las cifras decimales que aparecen en "precisix". La precisin podr ser
 modificada con posterioridad a travs de Excel. Por lo que interesa
 que precisix sea grande.


 }

 var cade: TcadNumero;

Begin
  write(ft, sepacsv);

  if ABS( FRAC(r) ) > 0
  then
      str(r:0:precisix, cade)
  else
      str(r:0:0, cade);

  if salto
  then writeln(ft, cade)
  else write(ft, cade);

End;
{ ************************************************ }


procedure escridaHT(var ft: text; r: Tnumero0; precis: byte);

{  Given a real number "r", this procedure writes it in an item of a table in
 the (open) HTML file "ft" acording to the following settings:
   "tabul": number of tabulator spaces;
   "precis": number of decimal digits;
   "tabulando": enter tabulator char?
   "salto": enter new line char?

  Dado
       ft = fichero de texto dispuesto para escribir en l,
       r  = nmero,    
   precis = nmero de cifras decimales (precisin),

 este procedimiento escribe el valor "r" en el fichero "ft" con slo
 las cifras decimales que aparecen en "precis", colocndolo dentro
 de una tabla de HTML con la directiva <td>.
 }



Begin
  write(ft, '<td align="right">');
  write(ft, r:0:precis);
  writeln(ft, '</td>');
End;
{ ************************************************ }



procedure escridaEEHT(var ft: text; r: Tnumero0; precis: byte);

{ Given a real number "r", this procedure writes it in a item of a table in
 the (open) HTML file "ft" acording to the following settings:
   "tabul": number of tabulator spaces;
   "precis": number of decimal digits;
   "tabulando": enter tabulator char?
   "salto": enter new line char?
 It also detects the integer numbers being written without decimal digits.

  Dado
       ft = fichero de texto dispuesto para escribir en l,
       r  = nmero,   
   precis = nmero de cifras decimales (precisin),
    salto = salto a la lnea siguiente despues de escribir ?

 este procedimiento escribe el valor "r" en el fichero "ft" con slo
 las cifras decimales que aparecen en "precis", colocndolo dentro
 de una tabla HTML.

 }

  var cade: TcadNumero;

Begin
  write(ft, '<td align="right">');

  if ABS( FRAC(r) ) > 0
  then
      str(r:0:precis, cade)
  else
      str(r:0:0, cade);

  write(ft, cade);

  writeln(ft, '</td>');
End;
{ ************************************************ }

procedure escridatlist(var ft: text;  r: Tnumero0; precis: byte;
                       salto, final: boolean);
{ 
 Given a real number "r", this procedure writes it in a R list of
 the (open) source file "ft" acording to the following settings:
   "tabul": number of tabulator spaces;
   "precis": number of decimal digits;
   "tabulando": enter tabulator char?
   "salto": enter new line char?
   "final": Is "r" the last of the R list?

 Procedimiento para nmeros que puedan ser indistintamente reales
 o enteros. Si son enteros, le coloca .000
  Dado
       ft = fichero de texto dispuesto para escribir en l,
       r  = nmero,
  tabulando = introduzco caracter tabulador despues de escribir el valor?
    salto = salto a la lnea siguiente despues de escribir ?
    fin = fin de la lista?

 este procedimiento escribe el valor "r" en el fichero "ft" con 
 las cifras decimales que aparecen en "precisix". 

 }



Begin
  write(ft, r:0:precis);

  if not(final) then
   begin
     if salto then writeln(ft, ',')
              else write(ft, ',');
   end;

End;
{ ************************************************ }


procedure escridatEElist(var ft: text; r: Tnumero0; precis: byte;
                         salto, final: boolean);

{  Given a real number "r", this procedure writes it in a R list in 
 the (open) source file "ft" acording to the following settings:
   "tabul": number of tabulator spaces;
   "precis": number of decimal digits;
   "tabulando": enter tabulator char?
   "salto": enter new line char?
   "final": Is "r" the last of the R list?
 It also detects the integer numbers being written without decimal digits.

 Procedimiento para nmeros que puedan ser enteros.

  Dado
       ft = fichero de texto dispuesto para escribir en l,
       r  = nmero,
tabulando = introduzco caracter tabulador despues de escribir el valor?
    salto = salto a la lnea siguiente despues de escribir ?

 este procedimiento escribe el valor "r" en el fichero "ft" con slo
 las cifras decimales que aparecen en "precisix". 
 }

 var cade: TcadNumero;

Begin
  if ABS( FRAC(r) ) > 0
  then
      str(r:0:precis, cade)
  else
      str(r:0:0, cade);

  write(ft, cade);
  
  if not(final) then
   begin
     if salto then writeln(ft, ',')
              else write(ft, ',');
   end;

End;
{ ************************************************ }



function CheckPath(var cami: TcaminFi): boolean;
{
  Realiza dos cosas independientes:
1) es el camino "cami" vaco? y
2) comprueba que "cami" sea efectivamente un camino
}

  var d: byte;
Begin
  d:= length(cami);

  if d = 0 then
    begin
      CheckPath:= TRUE;
      exit;
    end;
  
  if (cami[d] <> '\') then cami:= cami + '\';
  CheckPath:= FALSE;  
End;
{ ******************************************************** }

procedure leeTituloFich(var cami: TcaminFi; var f: text);
{
Reading short title:
It reads the short title from f and saves it in 
cami+"\titulo.txt".
}

  var cad1: TcadenaTi;
      fn: text;
Begin
  readln(f, cad1);

  assign(fn, cami+'titulo.txt');    rewrite(fn);
  writeln(fn, cad1);    { Title }
  close(fn);
End;
{ ******************************************************** }

{
procedure showTitulo(var cami: TcaminFi; var f: text; cini: Tcadena1);

Showing short title:
It reads the short title from cami+"\titulo.txt" and writes it
in f

  var cad1: TcadenaTi;
      fn: text;
Begin
  assign(fn, cami+'titulo.txt');    reset(fn);
  readln(fn, cad1);
  close(fn);

  write(f, cini);
  writeln(f, cad1);
End;
}

procedure showTitulo(var cami: TcaminFi; var f: text; tipfich: char);
{
Showing short title:
It reads the short title from cami+"\titulo.txt" and writes it
in f

tipfich = 't', a text file
tipfich = 'h', a HTML file
tipfich = 'c', a CSV file
tipfich = 'r', a R source file
}

  var cad1: TcadenaTi;
      fn: text;
Begin
  assign(fn, cami+'titulo.txt');    reset(fn);
  readln(fn, cad1);
  close(fn);

  if tipfich= 'c'
  then write(f, '"')
  else if tipfich= 'r' then write(f, '# ');

  write(f, cad1);

  if tipfich= 'c' then write(f, '"');
  
  writeln(f);
End;


END.
