{ =====================================================
 Copyright (C) 2007 Francisco A. Ocana & Pablo Onate 

 This file is part of IndElec

 IndElec is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 IndElec is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You have received a copy of the GNU General Public License
 in a file called COPYING which you should have received with
 IndElec. If not, see <http://www.gnu.org/licenses/>.
 ===================================================== 
}
unit CONSDATO;

{
Routines involved in the process of creating the binary database, 
for electoral data with aggregation levels. 
Such a binary database is made by means of a 
step-by-step process, where the routines contained in this unit are used.
Among other capabilities, such routines do the following tasks:
- create the text files siglas.txt and siglaso.txt,
- enter the PANE/NO-PANE (non-state-wide/state-wide) information into
  the binary database.
}

INTERFACE

uses COMUN, DEFNIVEL;

function lecturaTXT(var infoFi: TOInfoFich): boolean;
{
 This function reads the text file infoFi.camin+infoFi.nom.dab and thus creates
the first version of the database infoFi.camin+'Partido.bda', which includes
the acronym, the votos and the seats for each party.

Construye la base de datos de partidos polticos a partir de un fichero
de texto con el siguiente formato:

  Ttulo-breve-descripcin
  No Niveles de agregacin tiles (No. de niveles de agregacin a considerar)
  Indice del nivel de agregacin mxima considerado en los datos
  Cdigo del item del nivel de agregacin mxima (Cdigo del item mximo)
  (fila en blanco)
  En filas aparecen, para cada parido,
  - los cdigos correspondientes a cada uno de los items de los distintos niveles
  - Sigla_partido
  - votos_partido
  - escaos_partido


Tngase presente que, si el cdigo del item del nivel de agregacin 
mxima (Cdigo_para_Agregacin_mxima) es 0,
entonces el programa interpreta que el cdigo
del item del nivel de agregacin mxima
estar especificado en cada uno de los datos de los
partidos (suele ser til, por ejemplo, en las autonmicas).
En caso contrario (si Cdigo_para_Agregracin_mxima no es 0), se utiliza
dicho cdigo para el nivel de agregacin mxima y, por tanto,
se supone que entre los datos de los
partidos no aparecerespecificado el cdigo del item de Agregacin
mxima.    }



function codato2(var camino: TcaminFi): boolean;
{
This function extracts all the party acronyms from the file "Partido.bda" and thus
creates the first version of the text file "siglas.txt" with such acronyms.
This version is conceived as a template to introduce the state/regional party profiles.

A partir del fichero "camino"+partido.bda  crea la versin preliminar
del fichero siglas.txt, que contiene la especificacin de partidos
PANEs y NO-PANEs. Una vez que finalice la ejecucin de este procedimiento,
el usuario deber especificar si los partidos de dicho fichero son
PANES o NO-PANE
}


function codato3(var camino: TcaminFi): boolean;
{
This function reads the state/regional party profiles, which are entered through the
text file "siglas.txt" and incorporates such information into the database.

Este procedimiento introduce la informacin contenida en "camino"+siglas.txt
en la base de datos "camino"+partido.bda .
}

function codato4(var camino: TcaminFi; var Mnivel: TMatrNivel): boolean;
{
This function creates the first version of the text file "siglaso.txt" ,
to define the equivalence classes of party acronyms in the electoral data. 
In such a template file, the acronyms used by political parties in the maximum 
level of aggregation are indentied.

Una vez ejecutados los procedimientos anteriores e inicializados los
cdigos de los niveles, este procedimiento crea la versin preliminar
del fichero de clases de partidos, denominado
              "camino"+siglaso.txt
}


IMPLEMENTATION



uses DIMSIGLA, COMUTIL;


function lecturaTXT(var infoFi: TOInfoFich): boolean;
{ 
 This function reads the text file infoFi.camin+infoFi.nom.dab and thus creates
the first version of the database of political parties, infoFi.camin+'Partido.bda', 
which includes the acronym, the votos and the seats for each party.

 Lee el fichero infoFi.camin+infoFi.nom.dab por primera vez y crea la base de
 datos en el fichero infoFi.camin+'partido.bda' . En dicha base de datos se habrn
 almacenado para cada item, correspondiente a cada nivel de agregacin considerado, 
 las candidaturas, los votos y los escaos obtenidos.      
}

  var f: text;
      vParti: TRegPartido;
      gPa: TfichRegPartido;
      i0ni, i: TItmsNivel;

Begin
  {$I-}
  assign(f, infoFi.camin + infoFi.nom+'.dab');        
  reset(f);
  {$I+}

  if IoResult <> 0 then
    begin
      lecturaTXT:= TRUE;
      exit;
    end;

  assign(gPa, infoFi.camin+'Partido.bda');   rewrite(gPa);

  vParti.PANE:= FALSE;

  leeTituloFich(infoFi.camin, f); { Read the title-description of data }
  readln(f, i);    { Number of aggregation levels }

  if i = 0 then
    begin
      lecturaTXT:= TRUE;
      close(f);  close(gPa);
      exit;
    end;

  vParti.NoNiveleUti:= Pred(i);

  readln(f, i);  { Get the maximum level of data aggregation }
  vParti.IndNiveAgrMax:= Pred(i);

{ Get the minimun level of data aggregation}
  vParti.NoNiveleUti:= vParti.IndNiveAgrMax + vParti.NoNiveleUti;
      

  readln(f, i0ni); { Code of the item of the maximum level of aggregation  }

  readln(f);

  if i0ni = 0
  then
      i0ni:= vParti.IndNiveAgrMax
  else
    begin
      vParti.CgClasifica[ vParti.IndNiveAgrMax ]:= i0ni;
      i0ni:= Succ(vParti.IndNiveAgrMax);
    end;

  repeat
    for i:= i0ni to vParti.NoNiveleUti do
          readln(f, vParti.CgClasifica[i]);

    leeCadenaLn(f, @(vParti.SiglasCandi), lenCandida);

    readln(f, vParti.votos);
    readln(f, vParti.escanos);
    readln(f);

    write(gPa, vParti);
  until Eof(f);

  close(f);
  close(gPa);
  lecturaTXT:= FALSE;
End;
{ ***************************************************************}

function ordens(var cad1, cad2: TcadCandi): boolean;
{ 
  This function returns 
     FALSE, when cad1 <= cad2  (alphabetic order)
     TRUE , when cad1 > cad2

  Esta funcin establece el orden entre dos candidaturas "cad1" y "cad2". 
  En concreto, devuelve
     FALSE, cuando cad1 <= cad2
     TRUE , cuando cad1 > cad2
 }
 
  var len, l: byte;
      mayorl: boolean;  { auxiliary }

Begin
  len:= length(cad1);
  l:= length(cad2);

  if l < len 
  then
    begin
      len:= l;    { minimum of l and len }
      mayorl:= TRUE;
    end
  else
    mayorl:= FALSE;
         

  for l:= 1 to len do
    begin
      if cad1[l] < cad2[l]
      then
        begin
          ordens:= FALSE;
          exit;
        end
      else
        begin
          if cad1[l] > cad2[l]   then
            begin
              ordens:= TRUE;
              exit;
            end;
        end;
    end;

  ordens:= mayorl;  { length(cad1) > length(cad2);}
End;
{ ********************************************************************** }

procedure ordenSiglas(pSigla: PVectSiglas; noSiglas: TRgSiglasc);
{
 It sorts the vector "pSigla" of "noSiglas" party acronyms.

  Ordena el vector de siglas de candidaturas vSigla= ( vSigla[k] : k = 0..noSiglas )
}

var ordenado: boolean;
      siglax: TRegSiglas;
      i: TRgSiglasc;

Begin
  Dec(noSiglas);

  repeat
   ordenado:= TRUE;

   for i:= 0 to noSiglas do
    if ordens(pSigla^[i].SiglasCandi, pSigla^[Succ(i)].SiglasCandi) then
     begin
       ordenado:= FALSE;

       siglax:= pSigla^[i];
       pSigla^[i]:= pSigla^[Succ(i)];
       pSigla^[Succ(i)]:= siglax;
     end;


  until ordenado;

 
End;

{ *******************************************************************}

function codato2(var camino: TcaminFi): boolean;
{
This function extracts all the party acronyms from the file "Partido.bda" and thus
creates the first version of the text file "siglas.txt" with such acronyms.
This version is conceived as a template to introduce the state/regional party profiles.

Esta funcin extrae del fichero "Partido.bda" todas las siglas de los distintos
partidos y, posteriormente, las almacena en el fichero "siglas.txt" en orden
creciente, asignando el caracter PANE o NO-PANE.
}


  label finBusqueda;

  var gPa: TfichRegPartido;
      vSigla: TVectSiglas;
      vParti: TRegPartido;
      f: text;
      conta, i: TRgSiglasc;


Begin
{$I-}
  assign(gPa, camino+'Partido.bda');   reset(gPa);
{$I+}

  if IoResult <> 0 then
    begin
      codato2:= TRUE;
      exit;
    end;

  conta:= 0;

  repeat
    read(gPa, vParti);

    i:= 0;

    while (i < conta) do
      begin
        if vParti.SiglasCandi = vSigla[i].SiglasCandi 
        then goto finBusqueda;
        
        Inc(i);
      end;

    Inc(conta);

    vSigla[i].SiglasCandi:= vParti.SiglasCandi;
    vSigla[i].PANE:= vParti.PANE;

    finBusqueda:
  until Eof(gPa);

  close(gPa);

  Dec(conta);  { Number (-1) of acronyms  }
  if (conta > 0) then ordenSiglas(@vSigla, conta);

  assign(f, camino+'siglas.txt');   rewrite(f);
  writeln(f, Succ(conta));

  for i:= 0 to conta do
    begin
      writeln(f, vSigla[i].SiglasCandi);

      if vSigla[i].PANE then writeln(f, 'PANE')
                        else writeln(f, 'NO-PANE');
      writeln(f);
    end;

  close(f);

  codato2:= FALSE;
End;
{ ******************************************************************}

function codato3(var camino: TcaminFi): boolean;
{
  This function reads the state/regional party profiles, which are entered through the
  text file "siglas.txt", and incorporates such information into the database.

  Esta funcin lleva a cabo la lectura del fichero "siglas.txt" y, ms concretamente, las
  asignaciones de partidos polticos dentro de las clases PANEs y NO-PANEs. Una vez leida
  dichas asignaciones, las incorpora a la base de datos del fichero "partido.bda"
}
  var f: text;
      pSigla: PVectSiglas;
      i, conta: TRgSiglasc;
      ca2: string[2];
      gPa, gPan: TfichRegPartido;
      vParti: TRegPartido;

Begin
  {$I-}
  assign(f, camino+'siglas.txt');   reset(f);
  {$I+}

  if IoResult <> 0 then
    begin
      codato3:= TRUE;
      exit;
    end;

  readln(f, conta);  

  GetMem(pSigla, sizeOf(TRegSiglas) * TlongNatural(conta)); 

  Dec(conta);
 
  for i:= 0 to conta do
    begin
      leeCadenaLn(f, @(pSigla^[i].SiglasCandi), lenCandida);
      leeCadenaLn(f, @ca2, 2);
      readln(f);

      ca2[1]:= upcase(ca2[1]);
      ca2[2]:= upcase(ca2[2]);

      if ca2 = 'NO' then pSigla^[i].PANE:= FALSE
                    else pSigla^[i].PANE:= TRUE;
    end;

  close(f);

  ordenSiglas(pSigla, conta);

  assign(gPa, camino+'Partido.bda');       reset(gPa);
  assign(gPan, camino+'Partidon.bda');     rewrite(gPan);

  repeat
    read(gPa, vParti);

    for i:= 0 to conta do
      if vParti.SiglasCandi = pSigla^[i].SiglasCandi then
        begin
          vParti.PANE := pSigla^[i].PANE;
          break;
        end;

    write(gPan, vParti);
  until Eof(gPa);

  close(gPa);   close(gPan);

  FreeMem(pSigla, sizeOf(TRegSiglas) * TlongNatural(Succ(conta)) );

  assign(gPa, camino+'Partido.bda');       erase(gPa);

  assign(gPan, camino+'Partidon.bda');
  rename(gPan, camino+'Partido.bda');

  codato3:= FALSE;
End;
{ *********************************************************** }

procedure iniDecisionMxLevAg(var camino: TcaminFi;
                      pSigla: PVectSiglas;
                      nosig: TRgSiglasc;
                      var nosigGlobal: TRgSiglasc;
                      var Mnivel: TMatrNivel);

{
  This procedure identifies the acronyms used by political parties in the
maximum level of aggregation. The number of such acronyms is returned in "nosigGlobal".

  Dados
    pSigla : puntero a TVectSiglas con las siglas de partidos
    nosig = nmero de siglas,

 realiza el estudio de las siglas que aparecen en los recuentos globales,
 devolvindolas en el campo PANE con TRUE.
}


 label otro;

  var
      gPa: TfichRegPartido;
      vParti: TRegPartido;
      j: TRgSiglasc;

Begin
  for j:= 0 to nosig do pSigla^[j].PANE:= FALSE;

  assign(gPa, camino+'Partido.bda');  reset(gPa);

  repeat
    read(gPa, vParti);

    for j:= Succ(vParti.IndNiveAgrMax) to vParti.NoNiveleUti do
     if vParti.CgClasifica[j] <> Mnivel[j]^[ Mnivel[j]^[0] ]
     then goto otro;

    for j:= 0 to nosig do
      if vParti.SiglasCandi = pSigla^[j].SiglasCandi then
        begin
          pSigla^[j].PANE:= TRUE;
          goto otro;
        end;

    otro:
  until Eof(gPa);

  close(gPa);

  nosigGlobal:= 0;

  for j:= 0 to nosig do
     if pSigla^[j].PANE then Inc(nosigGlobal);
End;



function codato4(var camino: TcaminFi; var Mnivel: TMatrNivel): boolean;
{ 
  This function creates the first version of the text file "siglaso.txt" ,
  to define the equivalence classes of party acronyms in the electoral data. 
  In such a template file, the acronyms used by political parties in the maximum 
  level of aggregation are indentied.

  Crea la versin preliminar del fichero "siglaso.txt" que contendr las clases de 
  equivalencia de siglas de partidos polticos. La idea subyacenta al crear
  la forma preliminar de "siglaso.txt" consiste en separar las siglas utilizadas en
  los recuentos finales (nivel de agregacin mxima) de las siglas consideradas en 
  los items de los niveles de agregacin que no son el de agregacin mxima. }

  var f: text;
      pSigla: PVectSiglas;
      i, conta: TRgSiglasc;

Begin
  {$I-}
  assign(f, camino+'siglas.txt');   reset(f);
  {$I+}

  if IoResult <> 0 then
    begin
      codato4:= TRUE;
      exit;
    end;

  readln(f, conta);  

  GetMem(pSigla, sizeOf(TRegSiglas) * TlongNatural(conta) ); 

  Dec(conta);

  for i:= 0 to conta do
    begin
      leeCadenaLn(f, @(pSigla^[i].SiglasCandi), lenCandida);

      readln(f);
      readln(f);
    end;

  close(f);

  ordenSiglas(pSigla, conta);

  iniDecisionMxLevAg(camino, pSigla, conta, i , Mnivel);

  assign(f, camino+'siglaso.txt');  rewrite(f);
  writeln(f, i);

  for i:= 0 to conta do
    if pSigla^[i].PANE then
      begin
        writeln(f, pSigla^[i].SiglasCandi);
        writeln(f);
      end;

  writeln(f);  writeln(f);
  for i:= 0 to 40 do write(f, '=');
  writeln(f);
  writeln(f, 'WARNING: the acronyms above appear in the highest');
  writeln(f, 'level of aggregation. Thus the following acronyms are');
  writeln(f, 'only associated to the rest of aggregation levels.');
  writeln(f, 'In the following list, an asterisk has been added to');
  writeln(f, 'each acronym only to distinguish them.');
  writeln(f);
  for i:= 0 to 40 do write(f, '=');
  for i:= 0 to 1 do writeln(f);

  for i:= 0 to conta do
    if NOT(pSigla^[i].PANE) then
      begin
        write(f, ' *  ');
        writeln(f, pSigla^[i].SiglasCandi);
        writeln(f);
      end;

  FreeMem(pSigla, sizeOf(TRegSiglas) * TlongNatural(Succ(conta)) );

  close(f);

  codato4:= FALSE;
End;

END.
