{ =====================================================
 Copyright (C) 2007 Francisco A. Ocana & Pablo Onate 

 This file is part of IndElec

 IndElec is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 IndElec is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You have received a copy of the GNU General Public License
 in a file called COPYING which you should have received with
 IndElec. If not, see <http://www.gnu.org/licenses/>.
 ===================================================== 
}
unit DEFNIVEL;

{
It is focused on the definition of the logic of the abstract level-based aggregation 
structure (discrete aggregation). This unit contains the 
definition of constants and data types to implement any
data aggregation structure.
}

INTERFACE

uses COMUN;

const
  NoMaxSiglasc = 400;    { Maximum number of acronyms in data for an election
        		 Some parties could have several acronyms.

        No. mximo de siglas de candidaturas en una eleccin, incluyendo
	las presentes en cada uno de los niveles y contadas tantas veces
	como aparezcan sus distintas versiones}

  NoMaxNivele = 4;    { Maximum number (-1) of aggregation levels }

  NoMaxItmNiv = 80;   { Maximum number (-1) of items in an aggregation level }

  lenCadItmNiv = 15;  { Length of strings naming the items of aggregation levels }

  EqNo = 6;   { Depends of  sizeOf(TRegNivel) }

  cNoDistritClases = 5;  {Number of Classes of Districts}

{ ===============================================================================
  EqNo is the minimum positive integer such that
    (EqNo+1)*sizeof(Tnumero) >= sizeOf(TRegNivel)

  Obtencin de "EqNo": no. (-1) de componentes de un vector array de tipo numero

  Nota: La idea de EqNo es definir un array[0..EqNo] of numero que contenga todos
  los elementos de un registro TRegNivel

 * Paso 1: Tamao en bytes de TRegNivel *

   IndNiveAgrMax (byte)			+1 
   NoNiveleUti (byte)			+1 
   CgClasifica: TClasificador		+5 [(NoMaxNivele+1)*sizeOf(TItmsNivel) =(4+1)*1= 5]
   clase (byte)				+1 
   posif (TPosiciFich)			+2 
   nopartif (TRgPartido)		+1 
   tamano (word)			+2 
   votos (numero)			+6 
   NomNivel (TcadItemNivel)		+16 [lenCadItmNiv+1 = sizeOf-TRegNivel =(15+1)= 16]

   	Suma real =  35 bytes    (suma anterior)
   	Suma efectiva = (Eqno+1)*6 = 42 bytes 

 * Equivalencia en nmero de datos del tipo "numero"

     sizeOf-TRegNivel / sizeOf(numero)  (42/6 = 7 )

   Restriccin: Suma real < Suma efectiva 

  =============================================================================== 
  Each item in a level are referred to as a code. The set of items of
  a level are contained in a (row) vector. In turn, each level is identified by
  a natural number in decreasing order of data disaggregation. Hence,
  the levels based structure can be viewed as something like a matrix but
  having row vectors with different dimensions.
  Example: Spanish elections
    total -  row 0
    CA    -  row 1
    Prov  -  row 2
    Distr -  row 3

  ===============================================================================
}


type

  TItmsNivel = byte;   
{ 	Range and codes of items of an aggregation level : 0..NoMaxItmNiv  }


  TClasificador = array[0..NoMaxNivele] of TItmsNivel;
{ 
  This array will code the items and will classify them into aggregation levels. 
  A higher component index, a lower level of aggregation is referred to.

  Nota: A mayor ndice del componente del vector TClasificador,  menor nivel de 
  agregacin. Como ejemplo, podra ser: 
     0=Total,  1=CA,   2=Prov,    3=Distr

  De entre los cdigos de los items suministrados por el usuario, el valor
  mximo no coincidir con ninguno de los que designan items. De esta forma,
  el cdigo mximo de los utilizados para los items de un nivel, servir para
  designar saturacin o suma de los items de dicho nivel de agregacin. 
} 

  

TcadItemNivel = string[lenCadItmNiv];
{ 
  String for naming the items of an aggregation level. 
}

  

TPosiciFich = word;
{ 
Position of the party data records for a given item in a binary data
file.

Posicin dentro del fichero de datos de los registros de datos de partidos
correspondientes a cierto item.
}
  
{ TRegPartido = record accounting for the electoral data about a party in
		a item. This record will contain the most complete information
		about a party in a item.  
}

  TRegPartido = record
    IndNiveAgrMax: byte; { Index of the highest level (-1) of aggregation  
			}
    NoNiveleUti: byte;
{      A pesar de su denominacin, NoNiveleUti contiene el ndice (-1) del
       nivel de agregacin mnima, de manera que se ha de cumplir que
          IndNiveAgrMax <= NoNiveleUti  .
}
    CgClasifica: TClasificador; 
{ 	Vector que contiene los cdigos que  clasifican a los 
	datos del partido en uno de los items, dentro del nivel considerado
}
    SiglasCandi: TcadCandi;  
{	Acronym - Siglas de la candidatura (del partido)
}
    PANE: boolean;	{ Is it a PANE party?  (PANE=regionalistic party) }
    votos,              { Number of votes }
    escanos: Tnumero;    { Number of seats }
  end;


  TfichRegPartido = file of TRegPartido;


{
Definition of a item of a level of aggregation
}
  TRegNivel = record
    IndNiveAgrMax: byte; { Index of the highest level (-1) of aggregation
                      	}
    NoNiveleUti: byte; 
{      Index of the lowest level (-1) of aggregation.
   
       A pesar de su denominacin, NoNiveleUti contiene el ndice (-1) del
       nivel de agregacin mnima, de manera que se ha de cumplir que
          IndNiveAgrMax <= NoNiveleUti  .
}
    CgClasifica: TClasificador; { Array of codes defining an item.
       Vector que contiene los cdigos que
       especifican el item dentro del rbol de niveles
       considerados. 
     }
    clase: byte;         { Classify the district according to its size }
    posif: TPosiciFich;  { Starting position for the data item in the data file *. bda
        posiposicin inicial para los datos del item en el fichero de datos *.bda
        }
    nopartif: TRgPartido; { Total number of parties in a item } 
    tamano: word;        {  Total number of seats }
    votos: Tnumero;       {  Total number of votes }
    NomNivel: TcadItemNivel;    { name of the considered item }
  end;


  TfichNivel = file of TRegNivel;




  TRegSiglas = record
    SiglasCandi: TcadCandi;  { Party acronym       }
    PANE: boolean;           { Is it a PANE party? }
  end;

  TVectSiglas = array[0..NoMaxSiglasc] of TRegSiglas;

  PVectSiglas = ^TVectSiglas;

  TRgSiglasc = word;  { Range for the whole set of party acronyms in an election
                        : 0...NoMaxSiglasc }


{    Vector for storing the codes of the items of an aggregation level  
  
     Caracterizador de los items contenidos en un nivel
     ===================================================    
}

  TVectItmNivel = array[0..NoMaxItmNiv] of TItmsNivel;
  PVectItmNivel = ^TVectItmNivel;

{   
    Matrix storing the code tree codes associated to the aggregation 
    structure. Each row is associated to an aggregation level.
    In each row, the maximum code integer stands for the collapse
    (the whole) of the corresponding aggregation level.
    ==================================================================   
}

  TMatrNivel = array[0..NoMaxNivele] of PVectItmNivel;



function ClaseDistrito(tamano: word): byte;



IMPLEMENTATION

function ClaseDistrito(tamano: word): byte;
{
This function clasifies the items of a level of aggregation according to
its size (total number of seats). It returns an index in the
range 1..cNoDistritClases

Notice:
cNoDistritClases = 5;  Number of Classes of Districts

Esta funcin clasifica los items en varias clases, dependiendo normalmente
del nmero de escaos. Est en principio pensada para el caso espaol.

}
Begin
  if (tamano < 2) then    { tamano = 1 }
    ClaseDistrito:= 1
    
  else if (tamano < 6) then    { tamano = 2 ... 5 }    
    ClaseDistrito:= 2     
   
  else if (tamano < 9) then   { tamano = 6..8 }
    ClaseDistrito:= 3
   
  else if (tamano < 31) then { tamano = 9 .. 30 }
    ClaseDistrito:= 4
   
  else
    ClaseDistrito:= 5;  {constant cNoDistritClases}
End;



END. 
