{ =====================================================
 Copyright (C) 2007 Francisco A. Ocana & Pablo Onate 

 This file is part of IndElec

 IndElec is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 IndElec is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You have received a copy of the GNU General Public License
 in a file called COPYING which you should have received with
 IndElec. If not, see <http://www.gnu.org/licenses/>.
 ===================================================== 
}

unit DIMAUXN;

{
Auxiliary routines for the module Dimensi. Such routines are
addressed to construct the files containing the data information
by aggregation levels.
}

INTERFACE
{$N+,E-,F+,O+}

uses COMUN, DIMVECTO, DEFNIVEL, DIMDEFN;

procedure GetMatriNivele(var camino: TcaminFi;
                         var NoNiveleUt, IndNivelMx: byte;
                         var Mnivel: TMatrNivel);

function RepasoCg(var camino: TcaminFi;
                  var NoNiveleUt, IndNivelMx: byte;
                  var Mnivel: TMatrNivel): boolean;

procedure finCg(var Mnivel: TMatrNivel);


procedure IniClasesParti(var camino: TcaminFi; var noClasParti: TRgPartido);

procedure ordenaDato4vot(Pa: PVecParty;  noParti: TRgPartido);

function iniDato1F(var camino: TcaminFi;
                   var vNivel: TRegNivel;
                   var AcPosiF: TposiciFich;
                   var gPan: TfichRegPartido): boolean;


procedure iniDato2F(var camino: TcaminFi; var Pa: PVecParty;
                  var vNivel: TRegNivel);

procedure IniNorma(var sumVot, sumEsc: Tnumero;
                 Pa: PVecParty;  ndat: TRgPartido);

procedure WriteNivel(var gout: Tfichero; var vNive: TRegNivel);

procedure ReadNivel(var gout: Tfichero; var vNive: TRegNivel);

function GetItm(var camino: TcaminFi;
                var vNivAmb: TRegNivel;
                IndAmb: byte;  iAmb: TItmsNivel): boolean;

procedure RemuFichCalc(var camino: TcaminFi);

procedure NuNiveleUtif(var NoNiveleUt, IndNivelAgrMx: byte;
                       var camino: TcaminFi);


IMPLEMENTATION

uses UDOS, DIMSIGLA;

{type Tcadena2 = string[3];
     TvectNuEq = array[0..EqNo] of numero;}


procedure busqueCg(Cg: TItmsNivel;  vVec: PVectItmNivel);
{
For a given item code (label) "Cg", this procedure checks if it is included in
the vector "vVec", which contains the codes of the items defining an aggregation level.

  Para el cdigo de item "Cg", comprueba si ste se encuentra en la lista
de cdigos de items almacenada ya en "vVec".  }

 var i, mt: TItmsNivel;

Begin
  mt:= vVec^[0];

  for i:= 1 to mt do
      if Cg = vVec^[i] then exit;

  Inc(mt);
  vVec^[0]:= mt;
  vVec^[mt]:= Cg;
End;
{ *************************************************** }

procedure ordenaCodesItm(vVec: PVectItmNivel);

{ 
This procedure sorts out the codes of the items in "vVec".

Lleva a cabo la ordenacin en orden creciente de todos los items
de un nivel almacenados en "vVec". }

  var i, jx, nx: TItmsNivel;
      ordenado: boolean;

Begin
  if (vVec = NIL) then exit;

  nx:= vVec^[0];

  if (nx < 2) then exit;  

  Dec(nx);

  repeat
    ordenado:= TRUE;

    for i:= 1 to nx do
      if vVec^[i] > vVec^[Succ(i)] then
        begin
          ordenado:= FALSE;
          jx:= vVec^[i];
          vVec^[i]:= vVec^[Succ(i)];
          vVec^[Succ(i)]:= jx;
        end;


  until ordenado;

End;
{ *************************************************** }

function IniRepaCg1(var camino: TcaminFi;
                    var NoNiveleUt, IndNivelMx: byte;
                    var Mnivel: TMatrNivel): boolean;
{
This function extracts the codes or labels of the items (geographical units) 
of the aggregation levels considered in the electoral data.

  Obtiene de la base de datos camino+"Partido.bda" los cdigos de los items
de cada uno de los niveles de agregacin. Para cada nivel de agregacin, ordena
los cdigos de los items, quedando el ltimo el cdigo de saturacin de
dicho nivel. Finalmente almacena dichos cdigos en el fichero
camino+"cgNivelo.aux" de la forma siguiente:

 NoNiveleUti | IndiNivelAgreMaxima | Mnivel[i]^[j] ,   j=0,..,Mnivel^[0] |
                                         ( Mnivel[i]^[0]+1 )

donde i=0..NoMaxNivele
}


  var vParti: TRegPartido;
      gPa: TfichRegPartido;
      gb: file of TItmsNivel;
      i: byte;
      j: TItmsNivel;

Begin
  {$I-}
  assign(gPa, camino+'Partido.bda');   reset(gPa);
  {$I+}

  if IoResult <> 0 then
    begin
      IniRepaCg1:= TRUE;
      exit;
    end;


  for i:= 0 to NoMaxNivele do
    begin
      {GetMem(Mnivel[i], Succ(NoMaxItmNiv) * sizeOf(TItmsNivel) );}
      new(Mnivel[i]);
      Mnivel[i]^[0]:= 0;
    end;

  
  repeat
    read(gPa, vParti);

    for i:= vParti.IndNiveAgrMax to vParti.NoNiveleUti do
          busqueCg(vParti.CgClasifica[i],  Mnivel[i]);

  until Eof(gPa);

  close(gPa);

  NoNiveleUt:= vParti.NoNiveleUti;
  IndNivelMx:= vParti.IndNiveAgrMax;

  for i:= 0 to NoMaxNivele do ordenaCodesItm(Mnivel[i]);


  assign(gb, camino+'cgNivelo.aux');  rewrite(gb);

  j:= NoNiveleUt;   write(gb, j);
  j:= IndNivelMx;   write(gb, j);

  for i:= 0 to NoMaxNivele do
    for j:= 0 to Mnivel[i]^[0] do write(gb, Mnivel[i]^[j]);

  close(gb);

  for i:= 0 to NoMaxNivele do dispose(Mnivel[i]);
    {FreeMem(Mnivel[i], Succ(NoMaxItmNiv) * sizeOf(TItmsNivel) );}
  IniRepaCg1:= FALSE;
End;
{ *************************************************** }


procedure GetMatriNivele(var camino: TcaminFi;
                         var NoNiveleUt, IndNivelMx: byte;
                         var Mnivel: TMatrNivel);
{
  This procedure reads the labels of items from cgNivelo.aux and then loads them
to the matrix "Mnivel".

  Lee los cdigos de los items relativos a los distintos niveles de agregacin 
presentes en los datos, los cules estn almacenados en el fichero
camino+"cgNivelo.aux". Adems de leer dichos cdigos, los almacena en
la matriz dinmica "Mnivel" por filas, asignando una fila para cada nivel
de agregacin considerado.  Adems de crear la matriz Mnivel, asigna
los correspondientes valores a "NoNiveleUt" y "IndNivelMx".
}

  var gb: file of TItmsNivel;
      i: byte;
      j, k: TItmsNivel;
      
Begin
  assign(gb, camino+'cgNivelo.aux');  reset(gb);

  read(gb, j);    NoNiveleUt:= j;
  read(gb, j);    IndNivelMx:= j;

  for i:= 0 to NoMaxNivele do
    begin
      read(gb, j);

      if j = 0
      then
         Mnivel[i]:= NIL
      else
        begin
          GetMem(Mnivel[i], Succ(j) * sizeOf(TItmsNivel));
          Mnivel[i]^[0]:= j;
          for k:= 1 to j do read(gb, Mnivel[i]^[k]);
        end;
    end;

  close(gb);
End;
{ ***************************************************************** }

function RepasoCg(var camino: TcaminFi; var NoNiveleUt, IndNivelMx: byte;
                  var Mnivel: TMatrNivel): boolean;

{
  Realiza una lectura del fichero camino+'Partido.bda', reteniendo
los cdigos de los items correspondientes a cada uno de los niveles
de agregacin considerados en los datos proporcionados en "Partido.bda".

Posteriormente, son creados en "Mnivel" un vector fila para cada nivel de
agregacin. "Mnivel" es una matriz dinmica en la que cada fila tiene una
longitud distinta e igual al nmero de items del nivel + 1.
En cada vector fila aparecern los cdigos de los items
considerados, siendo el ltimo cdigo el de saturacin de dicho nivel
de agregacin.
 }

Begin
  if IniRepaCg1(camino, NoNiveleUt, IndNivelMx, Mnivel) then
    begin
      RepasoCg:= TRUE;
      exit;
    end;

  GetMatriNivele(camino, NoNiveleUt, IndNivelMx, Mnivel);

  RepasoCg:= FALSE;
End;
{ ************************************************************ }

procedure finCg(var Mnivel: TMatrNivel);
 var i: byte;
Begin
  for i:= 0 to NoMaxNivele do
    if (Mnivel[i] <> NIL) then
        FreeMem(Mnivel[i], Succ(Mnivel[i]^[0])*sizeOf(TItmsNivel));
End;
{ ************************************************************ }


procedure IniClasesParti(var camino: TcaminFi; var noClasParti: TRgPartido);
{
  This procedure counts out the equivalence classes of party acronyms defined
in the file "siglaso.txt".

  Devuelve el nmero de clases de partidos existentes en el fichero
"camino+siglaso.txt". Cada "clase" corresponde  al grupo de siglas distintas
que designan una misma formacin poltica. Es decir, una misma formacin
poltica puede adoptar distintas siglas como denominacin en
distintas regiones, definiendo todas esas siglas a la clase de equivalencia
dada por la formacin poltica.
}

  var f: text;
      c: string[2];
Begin
  assign(f, camino+'siglaso.txt');  reset(f);
  readln(f);

  noClasParti:= 0;

  repeat
    if leeCadenaLn(f, @c, 2) then Inc(noClasParti);
  until Eof(f);

  if c[0] = #0 then Dec(noClasParti);

  close(f);
End;
{ ********************************************************************}


function IsPartido(var vParti: TRegPartido; var vNivel: TRegNivel): boolean;
{
 This procedure checks whether the electoral data of a political party, given
by "vParti", has been recorded in the item "vNivel".

 Comprueba si el registro electoral "vParti" es un resultado electoral
 obtenido en el item vNivel.
}

  var i: byte;
      

Begin
  for i:= vParti.IndNiveAgrMax to vParti.NoNiveleUti do
    begin
      if vParti.CgClasifica[i] <> vNivel.CgClasifica[i] then
        begin
          IsPartido:= FALSE;
          exit;
        end;
    end;

  IsPartido:= TRUE;
End;
{ ********************************************************************}

function iniDato1F(var camino: TcaminFi;  var vNivel: TRegNivel;
              var AcPosiF: TposiciFich;  var gPan: TfichRegPartido): boolean;

{
The data for a given item described by "vNivel" are drawn from the
electoral database, and then are saved in the file referred to "gPan".

  Dado un item descrito por "vNivel", extrae del fichero "camino+Partido.bda"
los datos electorales para dicho item, almacenndolos en un fichero aparte
auxiliar denominado
    camino+'Partidon.bda',
que habra sido asignado previamente a gPan.
Adems, devuelve en AcPosiF el nmero de registros de TRegPartido almacenados
en el fichero gPan despues de cada ejecucin. Adems, 
actualiza la informacin de los datos para el item
vNivel, almacenando los valores vNivel.noPartif y vNivel.posif.

Devuelve TRUE si efectivamente existen datos de partidos correspondientes al 
item especificado por el registro vNivel y FALSE, en caso contrario.
}

var vParti: TRegPartido;
    gPa: TfichRegPartido;

Begin
  assign(gPa, camino+'Partido.bda');     reset(gPa);
  

  vNivel.noPartif:= 0;

  repeat
    read(gPa, vParti);

    if (vParti.votos > 0) then
     if IsPartido(vParti, vNivel)  then     
      begin
        write(gPan, vParti);
        Inc(vNivel.noPartif);
      end;

  until Eof(gPa);

  close(gPa);  

  if vNivel.noPartif > 0
  then
    begin
      vNivel.posif:= AcPosiF;
      AcPosiF:= AcPosiF + vNivel.noPartif; {position to the following set of data}

      Dec(vNivel.noPartif);
      iniDato1F:= TRUE;
    end
  else
     iniDato1F:= FALSE;

End;
{ ********************************************************************}


procedure ordenaDato4vot(Pa: PVecParty;  noParti: TRgPartido);

{
  Ordena el vector al que apunta Pa en orden decreciente deacuerdo
  al nmero o al porcentaje de votos.
}

var  ordenado: boolean;
     i: TRgPartido;
     rx: Tparty;

Begin
  if noParti = 0 then exit;

  repeat
    ordenado:= TRUE;

    for i:= 1 to noParti do
      if ( Pa^[ Pred(i) ].Vp < Pa^[i].Vp ) then
        begin
          ordenado:= FALSE;

          rx:= Pa^[ Pred(i) ];
          Pa^[ Pred(i) ] := Pa^[i];
          Pa^[i] := rx;
        end;

  until ordenado;

End;
{ ************************************************************** }


procedure iniDato2F(var camino: TcaminFi; var Pa: PVecParty;  
                   var vNivel: TRegNivel);
{  
This procedure extracts the party data records from camino+'Partido.bda'. 
They are loaded into "Pa", where "noParti"+1 is 
the number of such records.

A partir de los datos electorales almacenados en el fichero camino+'Partido.bda'
de partidos polticos, construye en Pa un puntero a un vector 
con los partidos correspondientes al item vNivel, devolviendo en noParti (+1)
el nmero de partidos (-1) asociados. Se supone que, en el fichero camino+'Partido.bda',
los datos de los partidos de un mismo item aparecen de forma correlativa.
Asimismo, almacena en camIndElec+'Partido.aux' una copia del vector referenciado por Pa.
}

  var i: TRgPartido;
      vParti: TRegPartido;
      gPa: TfichRegPartido;

Begin
  dimVectParty(Pa, vNivel.nopartif);

  assign(gPa, camino+'Partido.bda');  reset(gPa);
  seek(gPa, vNivel.posif);

  for i:= 0 to vNivel.nopartif do
    begin
      read(gPa, vParti);

      Pa^[i].Vp:= vParti.votos;
      Pa^[i].Ep:= vParti.escanos;
      Pa^[i].PANE:= vParti.PANE;
      Pa^[i].SiglasCandi:= vParti.SiglasCandi;
    end;

  close(gPa);
End;

{ ************************************************************** }

procedure IniNorma(var sumVot, sumEsc: Tnumero;
                Pa: PVecParty;  ndat: TRgPartido);
{
 This procedure calculates the totals of votes and seats and the shares of
votes and seats, for each political party. 

 Normaliza  a porcentajes el contenido de "Pa", que contendr los datos
 electorales correspondientes a una cierta zona. Asimismo, asigna
    sumVot := suma total de votos
    sumEsc := suma total de escaos
 En el caso de que sumEsc = 0, entonces el valor del campo "D" no tiene
 por qu tener sentido.
}

  var i: TRgPartido;
      sumx: Tnumero;
Begin
  sumVot:= 0;   sumEsc:= 0;

  for i:= 0 to ndat do
    begin
      sumVot:= sumVot + Pa^[i].Vp;
      sumEsc:= sumEsc + Pa^[i].Ep;
    end;

{ Obtaining percentages of votes 
}  
  sumx:= sumVot*0.01;
  for i:= 0 to ndat do Pa^[i].Vp:= Pa^[i].Vp/sumx;

{ Obtaining percentages of seats 
}
  if sumEsc > 0 then
    begin
     sumx:= sumEsc * 0.01;
     for i:= 0 to ndat do Pa^[i].Ep:= Pa^[i].Ep/sumx;
    end;

End;
{ ***********************************************************++ }

procedure WriteNivel(var gout: Tfichero; var vNive: TRegNivel);

  var vNiveEq: array[0..EqNo] of Tnumero;
{    Vector de constantes reales equivalente en tamao (bytes)
                 a TRegNivelEq }


Begin
  FillChar(vNiveEq, Succ(EqNo) * sizeOf(Tnumero), 0 );
  move(vNive, vNiveEq, sizeOf(TRegNivel) );
  BlockWrite(file(gout), vNiveEq, Succ(EqNo));
End;
{ ********************************************************************}

procedure ReadNivel(var gout: Tfichero; var vNive: TRegNivel);

  var vNiveEq:array[0..EqNo] of Tnumero;
              {  Vector de constantes reales equivalente en tamao (bytes)
                 a TRegNivelEq }

Begin
  BlockRead(file(gout), vNiveEq, Succ(EqNo));
  move(vNiveEq, vNive, sizeOf(TRegNivel));
End;
{ ********************************************************************}

function GetItm(var camino: TcaminFi;  var vNivAmb: TRegNivel;
                IndAmb: byte;  iAmb: TItmsNivel): boolean;

{
For a given level of aggregation, "IndAmb", and one of its items, "iAmb", 
this function returns the record "vNivAmb" containing the information of
such an item.

  Supongamos que, para un cierto mbito de agregacin "IndAmb", se dispone
del fichero "camino"+nombreAmbito.cg, en el que estn incluidos los
registros de todos sus items. Esta funcin lleva a cabo la lectura
del item "iAmb" -simo del nivel de agregacin "IndAmb", 
devolviendo su contenido en "vNivAmb".
}

  var fniv: TfichNivel;
      nombAmb: TnombreFi;

Begin
  tomaNombreNivel(nombAmb, IndAmb);

  assign(fniv, camino+nombAmb+'.cg');  reset(fniv);
  read(fniv, vNivAmb);

  if vNivAmb.tamano < iAmb
  then
      GetItm:= FALSE
  else
    begin
      seek(fniv, Succ(iAmb) );
      read(fniv, vNivAmb);
      GetItm:= TRUE;
    end;

  close(fniv);
End;
{  ************************************************************ }

procedure RemuFichCalc(var camino: TcaminFi);
{
Remove temporary files: *.cg, *.aux, *.reg, *.dd.

Este procedimiento tiene por misin eliminar del directorio "camino"
todos los ficheros de clculos que puedan existir. En general, dichos
ficheros son caracterizados por su extensin. Dichos ficheros son
aquellos que tienen extensiones *.cg, *.aux, *.reg, *.dd
}

Begin
  Remueve(camino, 'cg');
  Remueve(camino, 'aux');
  Remueve(camino, 'reg');
  Remueve(camino, 'dd');
End;
{************************************************}

procedure NuNiveleUtif(var NoNiveleUt, IndNivelAgrMx: byte;
                       var camino: TcaminFi);

   var i: TItmsNivel;
       gb: file of TItmsNivel;
Begin
  assign(gb, camino+'cgNivelo.aux');  reset(gb);
  read(gb, i);    NoNiveleUt:= i;
  read(gb, i);    IndNivelAgrMx:= i;
  close(gb);
End;


END.
