{ =====================================================
 Copyright (C) 2007 Francisco A. Ocana & Pablo Onate 

 This file is part of IndElec

 IndElec is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 IndElec is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You have received a copy of the GNU General Public License
 in a file called COPYING which you should have received with
 IndElec. If not, see <http://www.gnu.org/licenses/>.
 ===================================================== 
}
unit DIMENSIU;

{
 Clculos finales del mdulo dimensi sin el regionalismo para datos con niveles de agregacin.

 Manager of the part module Dimensi of IndElec associated to only one
 electoral distribution. The regionalism and party linkage are jointly managed
 by another unit (dimReg.pas).

}

INTERFACE

uses COMUN, DIMDEFN;

procedure NombraFichDim(var gout: Tfichero; var camino: TcaminFi;
                        IndAmb: byte);

procedure NombraFichPartyD(var gPou: TfichParty; var camino: TcaminFi;
                           IndAmb: byte);

procedure pDimDNivel(var camino: TcaminFi;
                     NoNiveleUt, IndNivelMx: byte);

procedure pDimension(var camino: TcaminFi;
                     NoNiveleUt, IndNivelMx: byte);


IMPLEMENTATION

uses defNivel, dimSigla, dimAuxN, dimMedN;


procedure introPartidos(var gP: TfichParty;  Pa: PVecParty;
                        noParti: TRgPartido);
{
This procedure saves the "noParti"+1 party records in "Pa" into
the file referred to as "gP". Notice that an initial party record
is saved.
}

 var Pa0: Tparty;
     i: TRgPartido;

Begin
  Pa0.Vp:= 0;
  Pa0.Ep:= noParti;

  write(gP, Pa0);
  
  for i:= 0 to noParti do write(gP, Pa^[i]);
End;
{ *******************************************************************  }


procedure pdimensi(var gout: Tfichero; var gPou: TfichParty;
                var camino: TcaminFi; var vNive: TRegNivel);

{
Given
      gout = a D*.dd file of results (initialized for writing),
      gPou = a P*.dd file of political party records (initialized for writing),
    camino = a data path,
     vNive = an item record,
 IndiNivel = an aggregation level,

this procedure extracts the electoral data of the item "vNive", updates the information
of such an item record, calculates the indices of disproportionality and voting dimensions, and
saves the data of the considered political partiy records.
--------------------------------
   Dados
      gout = fichero (D*.dd) de resultados inicializado para escribir en l,
      gPou = fichero de partidos (P*.dd) inicializado para escribir en l,
    camino = directorio de datos,
     vNive = record con informacin de un item de un cierto nivel de agregacin,
 IndiNivel = ndice que designa el nivel de agregacin considerado,

 Este procedimiento extrae los datos electorales correspondientes al item designado por vNive,
 actualiza la informacin sobre el item (no. de votos, no. de escaos), calcula los ndices
 de desproporcionalidad y las dimensiones y, finalmente, lo almacena en el fichero
 gout. Asimismo, almacena los datos de los partidos polticos utilizados.

}

  var {noParti: TRgPartido;}
      Pa: PVecParty;
      totv, tote: Tnumero;


Begin
  {noParti:= vNive.nopartif;}
  iniDato2F(camino, Pa, vNive);
  ordenaDato4vot(Pa, vNive.nopartif);

  IniNorma(totv, tote, Pa, vNive.nopartif);

  vNive.tamano:= trunc(tote);
  vNive.Clase:= ClaseDistrito(vNive.tamano);
  vNive.votos:= totv;

  WriteNivel(gout, vNive);

  Medidas(camino, gout, Pa, vNive.nopartif, totv, tote);

  IntroPartidos(gPou, Pa, vNive.nopartif);

  delVectParty(Pa, vNive.nopartif);
{ FreeMem(Pa, TlongNatural(sizeOf(Tparty))* Succ(vNive.nopartif) );}
End;
{ ************************************************************** }

procedure NombraFichDim(var gout: Tfichero; var camino: TcaminFi;
                       IndAmb: byte);

  var nomx: TnombreFi;
Begin
  str(IndAmb, nomx);   nomx:= 'D' + nomx;

  assign(gout, camino+nomx+'.dd');
End;
{ *************************************************************** }


procedure NombraFichPartyD(var gPou: TfichParty; var camino: TcaminFi;
                           IndAmb: byte);

  var nomx: TnombreFi;
Begin
  str(IndAmb, nomx);   nomx:= 'P' + nomx;

  assign(gPou, camino+nomx+'.dd');
End;
{ *************************************************************** }


function iniNivelAux(var fniv: TfichNivel; var camino: TcaminFi;
                     var vNiv: TRegNivel;  IndNivel: byte): boolean;
{
 Toma el registro inicial del fichero camino+nombNivelAgregacion+'.cg' y lo
 asigna a vNiv. Asimismo, crea para escribir en l, el fichero 
 camino+nivel.aux, que contendr los registros de los items en el transcurso
 del clculo de las dimensiones.
}                     
                     
  var nomb: TnombreFi;

Begin
  tomaNombreNivel(nomb, IndNivel);

  {$I-}
  assign(fniv, camino+nomb+'.cg');
  reset(fniv);
  {$I+}

  if IoResult <> 0 then
    begin
      IniNivelAux:= TRUE;
      exit;
    end;

  read(fniv, vNiv);
  close(fniv);

  assign(fniv, camino+'nivel.aux');    rewrite(fniv);
  write(fniv, vNiv);

  IniNivelAux:= FALSE;
End;


procedure finNivelAux(var fniv: TfichNivel; var camino: TcaminFi;
                      IndNivel: byte);

  var
      nomb: TnombreFi;

Begin
  tomaNombreNivel(nomb, IndNivel);
  assign(fniv, camino+nomb+'.cg');   erase(fniv);

  assign(fniv, camino+'nivel.aux');
  rename(fniv, camino+nomb+'.cg');
End;


procedure finNivelAuxPP(var fniv: TfichNivel; var camino: TcaminFi;
                      IndNivel: byte; var vNiv: TRegNivel;
                      noitems: TItmsNivel);

  var
      nomb: TnombreFi;
      fnivx: TfichNivel;

Begin
  tomaNombreNivel(nomb, IndNivel);

  assign(fnivx, camino+nomb+'.cg');  

  if noitems = 0 
  then
    begin
      erase(fnivx);
      close(fniv);
    end
  else
    begin
      reset(fnivx);
      read(fnivx, vNiv);
      close(fnivx);

      vNiv.tamano:= Pred(noitems);
   
      assign(fnivx, camino+nomb+'.cg');   erase(fnivx);

      seek(fniv, 0);
      write(fniv, vNiv);
      close(fniv);
   
      assign(fniv, camino+'nivel.aux');
      rename(fniv, camino+nomb+'.cg');
    end;

End;
{ ************************************************************** }


procedure pDoNive(var camino: TcaminFi;  IndiNivel: byte;
                  var AcPosiF: TPosiciFich; var gPan: TfichRegPartido);

{
  Dados
          "camino" = camino de trabajo,
       "IndiNivel" = Indice del nivel de agregacin,

  Este procedimiento va aadiendo los datos de forma ordenada en camino+'partidon.bda'
  para cada uno de los items con datos del nivel de agregacin "IndiNivel". 

}

  var fniv: TfichNivel;
      iv, conta: TItmsNivel;
      vNiv: TRegNivel;    
      

Begin
  if iniNivelAux(fniv, camino, vNiv, IndiNivel) then exit;

  iv:= 0;
  conta:= 0;
  
 
  while GetItm(camino, vNiv, IndiNivel, iv) do
    begin
      if iniDato1F(camino, vNiv, AcPosiF, gPan) then
        begin
          Inc(conta);
          write(fniv, vNiv);
        end;

      Inc(iv);
    end;
  
  finNivelAuxPP(fniv, camino, IndiNivel, vNiv, conta);      
End;
{ ****************************************+     }

procedure pDimDNivel(var camino: TcaminFi;
                     NoNiveleUt, IndNivelMx: byte);

{ Crea el fichero 
    camino+'Partidon.bda' en el que aparecen los datos "ordenados" segn
el item y el nivel de agregacin. Al final, el fichero resultante
es renombrado con camino+'Partido.bda'.
   }


  var IndiNivel: byte;
      AcPosiF: TPosiciFich;
      gPan: TfichRegPartido;

Begin
  AcPosiF:= 0;
  assign(gPan, camino+'Partidon.bda');    rewrite(gPan);
  
  for IndiNivel:= IndNivelMx to NoNiveleUt do
         pDoNive(camino, IndiNivel, AcPosiF, gPan);

  close(gPan);

  assign(gPan, camino+'Partido.bda');   erase(gPan);

  assign(gPan, camino+'Partidon.bda');
  rename(gPan, camino+'Partido.bda');
End;

{ ****************************************+     }



procedure pDimNive(var camino: TcaminFi; IndiNivel: byte);

{
  Dados
          "camino" = camino de trabajo,
       "IndiNivel" = Indice del nivel de agregacin,

 este procedimiento calcula los ndices de desproporcionalidad y de las
 dimensiones de los partidos para cada uno de los items del nivel de
 agregacin designado por IndiNivel.
 Estos clculos son almacenados en un fichero denominado

          'D' + IndiNivel + '.dd'

 con el siguiente formato. Al principio del fichero aparecen 2 registros

    No-Items-considerados | IndiNivel ||

 luego, para cada item, se almacenan los registros

    vNive (EqNo+1) ||  valores-Despro-Dimen ( NNIndexs ) ,

 siendo NNIndexs el no. de medidas calculadas (ndices de desproporcionalidad
 y dimensiones de partidos = 48)

 De igual forma que en el fichero 'D' + IndiNivel + '.dd', se crea
 el fichero fichParty denominado
          'P'+IndiNivel + '.dd' ,
 almacenndose en l los datos de los partidos polticos considerados
 en cada uno de los items. De hecho, para cada item, se almacenan

 vParty0 (con el nmero de partidos (-1)) || registro-Tparty  (noParti+1)

}

  var fniv: TfichNivel;
      iv: TItmsNivel;
      gout: Tfichero;
      gPou: TfichParty;
      vNiv: TRegNivel;
      r: Tnumero;


Begin
  if iniNivelAux(fniv, camino, vNiv, IndiNivel) then exit;

  NombraFichDim(gout, camino, IndiNivel);
  rewrite(gout);

  r:= 0;       write(gout, r);
  iv:= 0;

  r:= IndiNivel;  write(gout, r);

  NombraFichPartyD(gPou, camino, IndiNivel);
  rewrite(gPou);

  while GetItm(camino, vNiv, IndiNivel, iv) do
    begin
      pdimensi(gout, gPou, camino, vNiv);

      write(fniv, vNiv);
      Inc(iv);
    end;

  close(fniv);
  close(gPou);

  if iv = 0
  then
    begin
      close(gout);

      NombraFichDim(gout, camino, IndiNivel);
      erase(gout);

      NombraFichPartyD(gPou, camino, IndiNivel);
      erase(gPou);
    end
  else
    begin
      r:= Pred(iv);
      seek(gout, 0);
      write(gout, r);

      close(gout);

      finNivelAux(fniv, camino, IndiNivel);
    end;
End;
{ ****************************************+     }

procedure pDimension(var camino: TcaminFi;
                     NoNiveleUt, IndNivelMx: byte);
  var IndiNivel: byte;
      
Begin
  for IndiNivel:= IndNivelMx to NoNiveleUt do
         pDimNive(camino, IndiNivel);
End;


END.
