{ =====================================================
 Copyright (C) 2007 Francisco A. Ocana & Pablo Onate 

 This file is part of IndElec

 IndElec is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 IndElec is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You have received a copy of the GNU General Public License
 in a file called COPYING which you should have received with
 IndElec. If not, see <http://www.gnu.org/licenses/>.
 ===================================================== 
}
unit DIMESTIM;

{
Routines to perform the statiscal summaries included in the 
output files of IndElec with report style.
}

INTERFACE

uses COMUN, DEFNIVEL, DIMVECTO;

procedure IniEstimax(var gout: Tfichero; NoItms: TItmsNivel;
                NoRegIniFich: word; iniciMedidas, noMedidas: byte;
                NoDimenss: word);


procedure IniEstimaxSesgo(var gout: Tfichero;
                NoItms: TItmsNivel; NoRegIniFich, NoDimenss: word);


function IniEstiSel(var gout: Tfichero;
                NoItms: TItmsNivel; var NoItmsClase: TItmsNivel;
                NoRegIniFich: word;
                iniciMedidas, noMedidas: byte; NoDimenss: word;
                clase: byte): boolean;

procedure Estimar1(noMedidas: byte);

procedure Estimax(var med: PVector; var Rc: PMatrix;
                  NoItm: TItmsNivel; var gout: Tfichero;
                  NoRegIniFich: word;
                  iniciMedidas, noMedidas: byte;
                  NoDimenss: word);

procedure EstimaxPROB(var med: PVector; var Rc: PMatrix;
                  NoItm: TItmsNivel; var gout: Tfichero;
                  NoRegIniFich: word;
                  iniciMedidas, noMedidas: byte;
                  NoDimenss: word; VOTO: boolean);



procedure EstimaxSel(var med: PVector; var Rc: PMatrix;
                  NoItm: TItmsNivel; var NoItmClase: TItmsNivel;
                  var gout: Tfichero; NoRegIniFich: word;
                  iniciMedidas, noMedidas: byte; NoDimenss: word;
                  clase: byte);



procedure EstimaxSelPROB(var med: PVector; var Rc: PMatrix;
                  NoItm: TItmsNivel; var NoItmClase: TItmsNivel;
                  var gout: Tfichero; NoRegIniFich: word;
                  iniciMedidas, noMedidas: byte; NoDimenss: word;
                  clase: byte; VOTO: boolean);


IMPLEMENTATION

uses  DIMAUXN;

procedure IniEstimax(var gout: Tfichero; NoItms: TItmsNivel;
                NoRegIniFich: word;
                iniciMedidas, noMedidas: byte; NoDimenss: word);

{
This procedure creates a binary file 'data.aux' with the distributions of 
of the indices computed by IndElec. For each electoral index, the following figures
are saved

 || NoItms | x(0) | x(1) | ... | x(NoItms) ||

where NoItms is the number of values (sometimes it is the number of items) and x is an index.
If NoValores" = -1, then no data is available for such an index.

Notice that "NoItms" is not necessarily the number of items, because not available score
is returned for some indices (e.g. for indices of bias) under certain circumstances.
}

  var
      i: byte;
      w: TItmsNivel;
      A: PMatrix;
      gx: Tfichero;
      rx: Tnumero;

Begin
  Dec(noMedidas);
  NoRegIniFich:= NoRegIniFich + EqNo + IniciMedidas; 

  dimMatrix(A, noMedidas, NoItms);

  for w:= 0 to NoItms do
    begin
      seek(gout, NoRegIniFich);

      for i:= 0 to noMedidas do  read(gout, A^[i]^[w]);

      Inc(NoRegIniFich, NoDimenss);
    end;

  assign(gx, camIndElec+'data.aux');  rewrite(gx);

  for i:= 0 to noMedidas do
    begin
      rx:= NoItms;          write(gx, rx);
      BlockWrite(file(gx), A^[i]^, Succ(NoItms));
    end;

  close(gx);

  delMatrix(A, noMedidas, NoItms);
End;
{ ***************************************************************** }


procedure AlmaSesgAux(A: PMatrix;  noNiv: TItmsNivel;
                      var gx: Tfichero; indi: byte);
{
This procedure saves the "indi"-th index of bias in the matrix "A" 
(its "indi"-th row) into the binary file "gx". 
It is assumed that each index of bias is pairwise with an coefficient.
}

  var w, conta: TItmsNivel;
      rx: Tnumero;

Begin
  conta:= 0;
  indi:= indi shl 1;

  for w:= 0 to noNiv do
      if A^[Succ(indi)]^[w] > 0 then Inc(conta);

  if conta = 0 then
    begin
      rx:= -1;
      write(gx, rx);
      exit;
    end;

  Dec(conta);
  rx:= conta;
  write(gx, rx);

  for w:= 0 to noNiv do
    if A^[ Succ(indi) ]^[w] > 0 then write(gx, A^[indi]^[w]);
End;


procedure IniEstimaxSesgo(var gout: Tfichero;   NoItms: TItmsNivel;
                     NoRegIniFich, NoDimenss: word);


{
This procedure is similar to the procedure iniEstimaxSesgo but for the indices
of bias.

Crea un fichero denominado 'data.aux' con las distribuciones univariantes
de distintas medidas (noMedidas). Para cada medida, se almacena

 || NoValores | x(0) | x(1) | ... | x(NoValores) ||

Observacin: Cuando "NoValores" = -1, esto significa que no hay valores

 }

 const nnDstotx = nnDspro + nnDsRobu;
       nnSesgTT1 = nnSesgTT - 1;  { 13 }
       nnSesg1 = nnSesg - 1;      {  6 } 

  var
      i: byte;
      w: TItmsNivel;
      A: PMatrix;
      gx: Tfichero;


Begin
  NoRegIniFich:= NoRegIniFich + EqNo + 1+ nnDstotx;
            { NoRegIniFich + EqNo + 14 }

  dimMatrix(A, nnSesgTT1, NoItms);

  for w:= 0 to NoItms do
    begin
      seek(gout, NoRegIniFich);

      for i:= 0 to nnSesgTT1 do read(gout, A^[i]^[w]);

      Inc(NoRegIniFich, NoDimenss);
    end;

  assign(gx, camIndElec+'data.aux');  rewrite(gx);

  for i:= 0 to nnSesg1 do AlmaSesgAux(A, NoItms, gx, i);

  close(gx);

  delMatrix(A, nnSesgTT1, NoItms);
End;
{ ***************************************************************** }

function IniEstiSel(var gout: Tfichero; NoItms: TItmsNivel;
           var NoItmsClase: TItmsNivel;  NoRegIniFich: word;
           iniciMedidas, noMedidas: byte; NoDimenss: word;
           clase: byte): boolean;

{
It is similar to the procedure iniEstimax for a given type of districts.

Crea un fichero denominado 'data.aux' con las distribuciones univariantes
de distintas medias (noMedidas). Para cada medida, se almacena

 || NoValores | x(0) | x(1) | ... | x(NoValores) ||

Observacin: Cuando "NoValores" = -1, esto significa que no hay valores

 }

  var
      i: byte;
      w: TItmsNivel;
      posIni: word;
      A: PMatrix;
      gx: Tfichero;
      rx: Tnumero;
      rniv: TRegNivel;

Begin
  Dec(noMedidas);

  posIni:= NoRegIniFich + EqNo + IniciMedidas;

  dimMatrix(A, noMedidas, NoItms);

  NoItmsClase:= 0;

  for w:= 0 to NoItms do
    begin
      seek(gout, NoRegIniFich);
      ReadNivel(gout, rniv);

      if rniv.clase = clase then
        begin
          seek(gout, posIni);
          for i:= 0 to noMedidas do  read(gout, A^[i]^[NoItmsClase]);
          Inc(NoItmsClase);
        end;

      Inc(NoRegIniFich, NoDimenss);
      Inc(posIni, NoDimenss);
    end;


  if NoItmsClase = 0 then
    begin
      delMatrix(A, noMedidas, NoItms);
      IniEstiSel:= TRUE;
      exit;
    end;

  Dec(NoItmsClase);

  assign(gx, 'data.aux');  rewrite(gx);

  for i:= 0 to noMedidas do
    begin
      rx:= NoItmsClase;          write(gx, rx);
      BlockWrite(file(gx), A^[i]^, Succ(NoItmsClase) );
    end;

  close(gx);
  IniEstiSel:= FALSE;
  delMatrix(A, noMedidas, NoItms);
End;
{ ***************************************************************** }



procedure AnulaTr(S: PMatrix; ndim: word);

{   Dados
       "S" = matrix simtrica ya creada,
    "ndim" = dimensin de la matriz,

 asigna valor 0 a todos los elementos de "S"  ("ndim"+1)x("ndim"+1).

 }

 var i: word;

Begin
  for i:= 0 to ndim do
        FillChar(S^[i]^, TlongNatural(Succ(i)) * sizeOf(Tnumero) , 0);
End;
{ ******************************************************************** }

procedure ordenNume(L: PVector;  nuNiv: TItmsNivel);
{
It sorts the vector "L" (dimension= "nuNiv"+1).
}

  var i: TItmsNivel;
      ordenado: boolean;
      r: Tnumero;
Begin
  if nuNiv = 0 then exit;

  repeat
    ordenado:= TRUE;

    for i:= 1 to nuNiv do      
     if L^[ Pred(i) ] > L^[i] then
       begin
         ordenado:= FALSE;
         r:= L^[i];
         L^[i] := L^[ Pred(i) ];
         L^[Pred(i)] := r;
       end;
      

  until ordenado;
End;
{ *********************************************************** }

function asegura(var ix: integer; ndim: byte): boolean;

{
 Fuerza que el valor ix est comprendido en el rango 1..(ndim+1)
}

Begin
  if ix > 0
  then
    begin
      if ix > Succ(ndim)   then
        begin
          asegura:= TRUE;
          ix:= ndim;
          exit;
        end;
    end
  else
    begin
      asegura:= TRUE;
      ix:= 0;
      exit;
    end;

  asegura:= FALSE;
End;
{ ********************************************************* }

function fQa(Lv: PVector; ndim: byte; alfa: real): Tnumero;

  var  ix: integer;
       rx: Tnumero;
       px: real;

Begin
  if ndim = 0 then
    begin
      fQa:= Lv^[0];
      exit;
    end;

  { ndim+1    number of data }

  px:= (ndim +1) * alfa;
  ix:= round(px);
  if  ix < px then Inc(ix);

  if asegura(ix, ndim) then
    begin
      fQa:= Lv^[ix];
      exit;
    end;

  rx:= Lv^[ Pred(ix) ];

  px:= (ndim +2) * alfa ;
  ix:= trunc(px);
  if px < ix then Dec(ix);

  if asegura(ix, ndim) then
    begin
      fQa := Lv^[ix];
      exit;
    end;

  rx:= rx + Lv^[Pred(ix)];
  fQa:= 0.5*rx;
End;
{ ********************************************************* }

function fMe(Lv: PVector;  ndim: byte): Tnumero;

  var ix: word;
      rx: Tnumero;
Begin
  ix:= Succ(ndim);   { number of data }

  Inc(ix);
  rx:= Lv^[ Pred(ix shr 1) ];

  Inc(ix);
  rx:= rx + Lv^[ Pred( ix shr 1) ];

  fMe:= 0.5*rx;
End;
{ *****************************************************}

procedure Estimar1(noMedidas: byte);

{
It calculates the quartiles of each of the "noMedidas" distributions.
These 5 statistics are saved in the binary file "datum.aux" as follows

  X_min | Q_1 | Mediana | Q_3 | X_max ||

Toma el fichero "data.aux" y va leyendo las distribuciones unidimensionales
de cada una de las medidas almacenadas all. A continuacin, se crea
el fichero "datum.aux" en el que se van almacenando 5 valores

  X_min | Q_1 | Mediana | Q_3 | X_max ||

}

  var gx, gy: Tfichero;
      Lv: PVector;
      rx: Tnumero;
      crx: Tcadreal absolute rx;
      ndim: TItmsNivel;
      j: word;
      i: byte;

Begin
  Dec(noMedidas);

  assign(gx, camIndElec+'data.aux');   reset(gx);
  assign(gy, camIndElec+'datum.aux');  rewrite(gy);

  for i:= 0 to noMedidas do
    begin
      read(gx, rx);

      if trunc(rx) = -1 then
        begin
          crx:= '*';
          for j:= 0 to 4 do write(gy, rx);
          continue;
        end;

      ndim:= trunc(rx);

      dimVector(Lv, ndim);
      BlockRead(file(gx), Lv^, Succ(ndim));
      ordenNume(Lv, ndim);

      write(gy, Lv^[0]); {minimum}

      rx:= fQa(Lv, ndim, 0.25); {Q1 }
      write(gy, rx);

      rx:= fMe(Lv, ndim); {median}
      write(gy, rx);

      rx:= fQa(Lv, ndim, 0.75); {Q3}
      write(gy, rx);

      write(gy, Lv^[ndim]); {maximum}

      delVector(Lv, ndim);
    end;

  close(gx);
  close(gy);
End;
{ ******************************************************** }

procedure Estimax(var med: PVector; var Rc: PMatrix;
            NoItm: TItmsNivel; var gout: Tfichero; NoRegIniFich: word;
            iniciMedidas, noMedidas: byte; NoDimenss: word);

{ Uniform distribution}

  var 
      L: PVector;
      w: TItmsNivel;
      i, j: byte;
      ma, mR: Tnumero;

Begin
  med:= NIL;   Rc:= NIL;
  reset(gout);

  Dec(noMedidas);
  NoRegIniFich:= NoRegIniFich + EqNo + IniciMedidas;

  dimVectoR(med, noMedidas);
  dimVectoR(L, noMedidas);
  dimSimetri(Rc, noMedidas);

  seek(gout, NoRegIniFich);

  BlockRead(file(gout), med^, Succ(noMedidas));
  AnulaTr(Rc, noMedidas);

  for w:= 1 to NoItm do
    begin
      Inc(NoRegIniFich, NoDimenss);
      seek(gout, NoRegIniFich);

      BlockRead(file(gout), L^, Succ(noMedidas));

      for i:= 0 to noMedidas do
        begin
          ma:= (L^[i] - med^[i])/Succ(w);
          med^[i]:= med^[i] + ma;

          for j:= 0 to i do
            begin
              mR:= Rc^[i]^[j];
              mR:= mR - mR/Succ(w);
              Rc^[i]^[j] := (L^[j] - med^[j])*ma + mR;
            end;
        end;
    end;

  delVectoR(L, noMedidas);
End;
{ ******************************************************************* }

procedure EstimaxPROB(var med: PVector; var Rc: PMatrix;
            NoItm: TItmsNivel; var gout: Tfichero; NoRegIniFich: word;
            iniciMedidas, noMedidas: byte; NoDimenss: word;
            VOTO: boolean);


{ 
This is a weighted version of the procedure Estimax, where the distributions of
votes or seats can be considered as weights.

versin ponderada de Estimax.
Este procedimiento lleva a cabo la estimacin ponderada con la distribucin
de votos o escaos}

 label meSalgo;

  var 
      L, Pw: PVector;
      w: TItmsNivel;
      i, j: byte;
      ma, acW: Tnumero;
      posici: word;
      vNivex: TRegNivel;

Begin
  med:= NIL;   Rc:= NIL;

  Dec(noMedidas);
  dimVectoR(Pw, NoItm);

{ Get the distribution of votes or seats }
  reset(gout);
  posici:= NoRegIniFich;
  acW:= 0;

  for w:= 0 to NoItm do
    begin
      seek(gout, posici);
      ReadNivel(gout, vNivex);

      if VOTO 
      then Pw^[w] := vNivex.votos
      else Pw^[w] := vNivex.tamano;
       
      acW:= acW + Pw^[w];           
      Inc(posici, NoDimenss);
    end;

  if abs(acW) < 1e-10 then goto meSalgo;

  for w:= 0 to NoItm do Pw^[w]:= Pw^[w]/acW;
{ end:  Get the distribution of votes or seats }

  reset(gout);
  posici:= NoRegIniFich + EqNo + IniciMedidas;

  dimVectoR(med, noMedidas);
  dimVectoR(L, noMedidas);
  dimSimetri(Rc, noMedidas);

  seek(gout, posici);

  BlockRead(file(gout), med^, Succ(noMedidas));
  AnulaTr(Rc, noMedidas);
  acW:= Pw^[0];

  for w:= 1 to NoItm do
    begin
      Inc(posici, NoDimenss);
      seek(gout, posici);

      BlocKRead(file(gout), L^, Succ(noMedidas));
      acW:= acW+Pw^[w];

      for i:= 0 to noMedidas do
        begin
          ma:= (L^[i] - med^[i])*Pw^[w];
          med^[i]:= med^[i] + ma/acW;

          for j:= 0 to i do                                       
           Rc^[i]^[j] := Rc^[i]^[j] + (L^[j] - med^[j])*ma;             
        end;
    end;
 
  delVectoR(L, noMedidas);

  meSalgo:
  delVectoR(Pw, NoItm);
End;
{ ******************************************************************* }

{ ******************************************************************* }


procedure EstimaxSel(var med: PVector; var Rc: PMatrix;
                  NoItm: TItmsNivel; var NoItmClase: TItmsNivel;
                  var gout: Tfichero;  NoRegIniFich: word;
                  iniciMedidas, noMedidas: byte; NoDimenss: word;
                  clase: byte);

  var 
      L: PVector;
      w: TItmsNivel;
      i, j: byte;
      ma, mR: Tnumero;
      posIni: word;
      rniv: TRegNivel;

Begin
  med:= NIL;   Rc:= NIL;

  reset(gout);
  Dec(noMedidas);

  posIni:= NoRegIniFich + EqNo + IniciMedidas;

  dimVectoR(med, noMedidas);
  dimVectoR(L, noMedidas);
  dimSimetri(Rc, noMedidas);

  FillChar(med^, Succ(noMedidas)*TlongNatural(sizeOf(Tnumero)), 0);
  AnulaTr(Rc, noMedidas);
  NoItmClase:= 0;

  for w:= 0 to NoItm do
    begin
      seek(gout, NoRegIniFich);
      ReadNivel(gout, rniv);

      if rniv.clase = clase then
        begin
          seek(gout, posIni);
          BlockRead(file(gout), L^, Succ(noMedidas));
          Inc(NoItmClase);

          for i:= 0 to noMedidas do
            begin
              ma:= (L^[i] - med^[i])/NoItmClase;
              med^[i]:= med^[i] + ma;

              for j:= 0 to i do
                begin
                  mR:= Rc^[i]^[j];
                  mR:= mR - mR/NoItmClase;
                  Rc^[i]^[j] := (L^[j] - med^[j])*ma + mR;
                end;
            end;
        end;

      Inc(posIni, NoDimenss);
      Inc(NoRegIniFich, NoDimenss);
    end;

  Dec(noItmClase);
  delVectoR(L, noMedidas);
End;
{ ******************************************************************* }

procedure EstimaxSelPROB(var med: PVector; var Rc: PMatrix;
                  NoItm: TItmsNivel;  var NoItmClase: TItmsNivel;
                  var gout: Tfichero;  NoRegIniFich: word;
                  iniciMedidas, noMedidas: byte; NoDimenss: word;
                  clase: byte; VOTO: boolean);

 label meSalgo;

  var 
      L, Pw: PVector;
      w: TItmsNivel;
      i, j: byte;
      ma, acW: Tnumero;
      posIni: word;
      rniv: TRegNivel;

Begin
  med:= NIL;  Rc:= NIL;

  Dec(noMedidas);
  dimVectoR(Pw, NoItm);

{ Get the distribution of votes or seats }
  reset(gout);
  posIni:= NoRegIniFich;
  acW:= 0;
  NoItmClase:= 0;

  for w:= 0 to NoItm do
    begin
      seek(gout, posIni);
      ReadNivel(gout, rniv);

      if rniv.clase = clase
      then
        begin
          if VOTO
          then Pw^[w] := rniv.votos
          else Pw^[w] := rniv.tamano;

          acW:= acW + Pw^[w];
          Inc(NoItmClase);
        end
      else
          Pw^[w]:= -2;

      Inc(posIni, NoDimenss);
    end;  

  if abs(acW) < 1e-10 then goto meSalgo;  {if NoItmClase = 0}

  for w:= 0 to NoItm do
   if Pw^[w] > -1 then Pw^[w]:= Pw^[w]/acW;

{ end: Get the distribution of votes or seats }

  Dec(NoItmClase);

  posIni:= NoRegIniFich + EqNo + IniciMedidas;

  dimVectoR(med, noMedidas);
  dimVectoR(L, noMedidas);
  dimSimetri(Rc, noMedidas);

  FillChar(med^, Succ(noMedidas)*TlongNatural(sizeOf(Tnumero)), 0);
  AnulaTr(Rc, noMedidas);
  acW:= 0;

  for w:= 0 to NoItm do
    begin
      if Pw^[w] > -1 then
        begin
          seek(gout, posIni);
          BlockRead(file(gout), L^, Succ(noMedidas));
          acW:= acW + Pw^[w];

          for i:= 0 to noMedidas do
            begin
              ma:= (L^[i] - med^[i])*Pw^[w];
              med^[i]:= med^[i] + ma/acW;

              for j:= 0 to i do
                 Rc^[i]^[j] := Rc^[i]^[j] + (L^[j] - med^[j])*ma;

            end;
        end;

      Inc(posIni, NoDimenss);
    end;

  delVectoR(L, noMedidas);

  meSalgo:
  delVectoR(Pw, NoItm);
End;
{ ******************************************************************* }


{ The End of dimEstim }
END.
