{ =====================================================
 Copyright (C) 2007 Francisco A. Ocana & Pablo Onate 

 This file is part of IndElec

 IndElec is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 IndElec is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You have received a copy of the GNU General Public License
 in a file called COPYING which you should have received with
 IndElec. If not, see <http://www.gnu.org/licenses/>.
 ===================================================== 
}
unit DIMREG;

{
This units performs the calculation of the indices of the regionalism and 
the party linkage for a party system. These indices depend on two different 
levels of data aggregation.
}

INTERFACE

uses COMUN, DEFNIVEL;


procedure NombraFichReg(var gout: Tfichero; var camino: TcaminFi;
                IndAmbSup, IndAmbInf: byte;  var vNivAmbSup: TRegNivel;
                exten: TextenFi);
{
Given
   IndAmbSup : upper level of aggregation,
   IndAmbInf : lower level of aggregation,
   vNivAmbSup : an item of the upper level of aggregation,

this procedure assigns a binary file "gout" for regionalism and party linkage
indices, such as follows
  RIndAmbSup_IndAmbInf_i(vNivAmbSup)."exten"
}

procedure pRegionalismo(var camino: TcaminFi;  NoNiveleUt, IndNivelMx: byte);


function inflationMoKa(var ENP, ENPavg : Tnumero): Tnumero;
{ 
 The Moenius and Kasuya inflation rate of party system linkage. 
}

function inflationCox(var ENP, ENPavg : Tnumero): Tnumero;
{ 
  The Cox inflation rate of party system linkage. 
}

IMPLEMENTATION

uses DIMDEFN, DIMSIGLA, DIMVECTO,
     DIMAUXN;

function inflationMoKa(var ENP, ENPavg : Tnumero): Tnumero;
{ 
 Moenius and Kasuya inflation rate of party system linkage. 
}
Begin
  inflationMoKa:= 100*(ENP- ENPavg)/ENPavg;
End;


function inflationCox(var ENP, ENPavg : Tnumero): Tnumero;
{ 
  Cox inflation rate of party system linkage. 
}
Begin
  inflationCox:= 100*(ENP- ENPavg)/ENP;
End;




function fENP(Pa: PVecParty;  noParti: TRgPartido): Tnumero;
{ 
 Effective number of parties (Laakso & Tagepera electoral index) 
}

 var ac: Tnumero2;
     i: TRgPartido;

Begin
  ac:= 0;
  for i:= 0 to noParti do 
    ac:= ac + SQR(Pa^[i].Vp * 0.1);
  fENP:= 100/ac;
End;


procedure GetDistrAmbSup(var camino: TcaminFi;  var PaG: PVecParty; 
                      var totPANE, totENP: Tnumero;
                      var vNivAmbSup: TRegNivel);
{
This procedure gets the distribution "PaG" of electoral data for parties in a
given regional item "vNivAmbSup". It also returns the regionalist vote, "totPANE", 
and the effective number of parties, "totENP", for the obtained electoral distribution.
}
  var
      i: TRgPartido;

Begin
  {noPartiG:= vNivAmbSup.nopartif;}
  iniDato2F(camino,PaG, vNivAmbSup);

  iniNorma(totPANE, totENP, PaG, vNivAmbSup.nopartif); 
  {temporary values in totPANE and totENP }

  totPANE:= 0;
  for i:= 0 to vNivAmbSup.nopartif do
      if PaG^[i].PANE then totPANE:= totPANE + PaG^[i].Vp;

  totENP:= fENP(PaG, vNivAmbSup.nopartif);
End;
{ **************************************************************+ }

procedure NombraFichReg(var gout: Tfichero; var camino: TcaminFi;
                     IndAmbSup, IndAmbInf: byte; var vNivAmbSup: TRegNivel;
                     exten: TextenFi);
{
Given
   IndAmbSup : upper level of aggregation,
   IndAmbInf : lower level of aggregation,
   vNivAmbSup : an item of the upper level of aggregation,
   exten: the extension file (.aux, .reg)

this procedure assigns a binary file "gout" for regionalism and party linkage
scores, such as follows
  RIndAmbSup_IndAmbInf_i(vNivAmbSup).exten
}

  var nomx: TnombreFi;
      cadx: string[lenItmDigits];
Begin
  nomx:= 'R';
  str(IndAmbSup, cadx);     nomx:= nomx + cadx +'_';
  str(IndAmbInf, cadx);     nomx:= nomx + cadx +'_' ;

  str(vNivAmbSup.CgClasifica[IndAmbSup], cadx);
  nomx:= nomx + cadx;

  assign(gout, camino+nomx+'.'+exten);
End;
{ ************************************************************** }

function NotIncluidoItm(var vNivAmbSup, vNivAmbInf: TRegNivel;
                     IndAmbSup: byte): boolean;
{
This function checks whether the item "vNivAmbSup", belonging to the aggregation
level "IndAmbSup", contains  "vNivAmbInf". It returns FALSE when it holds.
}
  var l: byte;
Begin
{ for l:= vNivAmbSup.IndNiveAgrMax to IndAmbSup do
      if vNivAmbSup.CgClasifica[l] <> vNivAmbInf.CgClasifica[l] then
        begin
          NotIncluidoItm:= TRUE;
          exit;
        end;
  NotIncluidoItm:= FALSE; 
}
  l:= vNivAmbSup.IndNiveAgrMax;

  while (vNivAmbSup.CgClasifica[l] = vNivAmbInf.CgClasifica[l]) do
    begin
      Inc(l);
      if (l > IndAmbSup) then break;
    end;

  NotIncluidoItm:= (l <= IndAmbSup); { not(l > IndAmbSup) }
End;
{ ******************************************************************* }

procedure IniRegionx2(var camino: TcaminFi; var vNivAmbInf: TRegNivel;
                PaG: PVecParty; pD: PVector; noPartiG: TRgPartido; 
                var totPANE: Tnumero; var gout: Tfichero);
{
This procedure extracts the electoral data for a given item "vNivAmbInf", at the
lower level of aggregation, and calculates indices of regionalism and party linkage.
They compare the political system for such an item and that of the upper aggregation 
level with data given by "PaG" and "noPartiG".
}

  var {noPartiR: TRgPartido;}
      PaR: PVecParty;
      i, j: TRgPartido;
      ac, rENP: Tnumero;
      fs: text;

Begin
  {noPartiR:= vNivAmbInf.nopartif;}
  iniDato2F(camino,PaR, vNivAmbInf);
  iniNorma(ac, rENP, PaR, vNivAmbInf.nopartif); 
{  ac and rENP contain temporary values }

  for i:= 0 to noPartiG do pD^[i]:= 0;

  rENP:= fENP(PaR, vNivAmbInf.nopartif); 

  ac:= 0; { Regionalist vote at the lower level item}

  assign(fs, camino+'siglaso.txt');

  for j:= 0 to vNivAmbInf.nopartif do
    begin
      for i:= 0 to noPartiG do
       if mismaSigla(fs, PaG^[i].SiglasCandi, PaR^[j].SiglasCandi) then
          begin
            pD^[i] := pD^[i] + PaR^[j].Vp;
            break;
          end;

      if PaR^[j].PANE then ac:= ac + PaR^[j].Vp
    end;

  close(fs);
  delVectParty(PaR, vNivAmbInf.nopartif);

{ Regionalist vote at the lower level item// Indice de voto regional}
  write(gout, ac); 

{ Differentiated regionalist vote// Indice de voto regionalista diferenciado
  Differentiated regional vote// Indice del voto regional diferenciado (Paco)
  Effective number of vote at the lower level item//Nmero effectivo de partidos  ENP del item del nivel inferior
}

  ac:= ac - totPANE;    {Differentiated regionalist vote}
  write(gout, ac);

  ac:= 0;   { <- Differentiated regional vote (regional volatility) }
  for i:= 0 to  noPartiG do ac:= ac + ABS(pD^[i] - PaG^[i].Vp);
  ac:= 0.5*ac;        {Differentiated regional vote (regional volatility) }
  
  write(gout, ac, rENP);

  for i:= 0 to noPartiG do write(gout, pD^[i]);
End;
{ ******************************************************************  }

function GetItmsAI_in_AS(var camino: TcaminFi;
            IndAmbSup, IndAmbInf: byte; var vNivAmbSup: TRegNivel;
            var PaG: PVecParty;  var totPANE, totENP: Tnumero): boolean;

  label fine;

  var
      vNivAmbInf: TRegNivel;
      iAmbInf, cont: TItmsNivel;
      gout: Tfichero;
      rx: Tnumero;
      pD: PVector;

Begin
  dimVectoR(pD, vNivAmbSup.nopartif); {complement of PaG}

  iAmbInf:= 0;
  cont:= 0;

  while GetItm(camino, vNivAmbInf, IndAmbInf, iAmbInf) do
    begin
      if NotIncluidoItm(vNivAmbSup, vNivAmbInf, IndAmbSup) then goto fine;

      if cont = 0 then
        begin
          NombraFichReg(gout, camino, IndAmbSup, IndAmbInf, vNivAmbSup, 'aux');
          rewrite(gout);
          rx:= 0;    write(gout, rx);   { for the number of items in the lower level }

          write(gout, totPANE, totENP);

          write(gout, rx, totPANE); {a trick; the are positions for ENPavg y ENPavg-w. 
                          Ocupan los sitios de ENPavg y ENPavg-w }

          rx:= IndAmbSup;  write(gout, rx);
          writeNivel(gout, vNivAmbSup);

          rx:= IndAmbInf;  write(gout, rx);

          rx:= vNivAmbSup.nopartif;  write(gout, rx);
        end;

      Inc(cont);

      writeNivel(gout, vNivAmbInf);

      IniRegionx2(camino, vNivAmbInf, PaG, pD, vNivAmbSup.nopartif,
               totPANE, gout);

      fine:
      Inc(iAmbInf);
    end;

  delVectParty(PaG, vNivAmbSup.nopartif);
  delVectoR(pD, vNivAmbSup.nopartif);

  if cont = 0
  then
      GetItmsAI_in_AS:= FALSE
  else
    begin
      rx:= Pred(cont);
      seek(gout, 0);
      write(gout, rx);
      close(gout);

      GetItmsAI_in_AS:= TRUE;
    end;
End;
{ ***********************************************************   }


procedure introENP(var gin: Tfichero; var totENP, rENP: Tnumero);
  var Dx, rx: Tnumero;
Begin
  Dx:= totENP - rENP;   { component of the Chibber and Kollman index }
  write(gin, rENP, Dx);

  rx:= 100*Dx/totENP;  { component of the Cox measure }
  write(gin, rx);

  rx:= 100*Dx/rENP; { component of the Moenius and Kasuya measure }
  write(gin, rx);
End;
{ ***********************************************************   }


procedure Regional3x(var camino: TcaminFi; IndAmbSup, IndAmbInf: byte;
                     var vNivAmbSup: TRegNivel;  {noPartiG: TRgPartido;}
                     var totENP: Tnumero);
{
noPartiG:= vNivAmbSup.nopartif
}

  const EqNo0 = EqNo + 1+ nnRegi0Fi;
        EqNoitm = 3;                  { cardinal: VR,  VRtaD,  VRPaco }
        EqNoitmmx = EqNo+1+ EqNoitm;

  var med: PVector;
      ac, r, acPv, mENP, wENP: Tnumero;
      itm, conta: TItmsNivel;
      j: TRgPartido;
      gin, gout: Tfichero;
      rENP: Tnumero;
      vNivx: TRegNivel;


Begin
  NombraFichReg(gout, camino, IndAmbSup, IndAmbInf, vNivAmbSup, 'aux');
  reset(gout);

  read(gout, r);   conta:= trunc(r);  {number of items }

  seek(gout, EqNo0);

  dimVector(med, vNivAmbSup.nopartif);
  FillChar(med^, TlongNatural(sizeOf(Tnumero))*Succ(vNivAmbSup.nopartif), 0);
  acPv:= 0; 
  mENP:= 0;  wENP:= 0;

  for itm:= 0 to conta do
    begin
      ReadNivel(gout, vNivx);
      for j:= 0 to EqNoitm do read(gout, r); {At the end, r <- ENP }      

      acPv:= acPv + vNivx.votos;
      mENP := mENP + (r-mENP)/Succ(itm);
      wENP := wENP + (r-wENP)*(vNivx.votos/acPv);

      for j:= 0 to vNivAmbSup.nopartif do
        begin
          read(gout, r);
          med^[j] := med^[j] + r;
        end;
    end;

  Inc(conta);
  for j:= 0 to vNivAmbSup.nopartif do med^[j]:= med^[j]/conta;
  Dec(conta);


  reset(gout);

  NombraFichReg(gin, camino, IndAmbSup, IndAmbInf, vNivAmbSup, 'reg');
  rewrite(gin);

{ Escritura de los registros iniciales del fichero REG : 
        NoItems_considerados (1) | totPANE (1) | totENP (1)  
}
  for j:= 1 to 3 do
    begin
      read(gout, r);      write(gin, r);
    end;

{ Escritura de los registros: I (1) | Iw (1) | 
}
  for j:= 1 to 2 do read(gout, r);
  write(gin, mENP, wENP);


  for j:= 1 to EqNo0-5 do  {5 =3+2 }
    begin
      read(gout, r);      write(gin, r);
    end;

  { Nota: el 5 anterior es el total de registros anteriores 3+2 }


  for itm:= 0 to conta do
    begin
      for j:= 1 to EqNoItmmx do
        begin
          read(gout, r);   write(gin, r);
        end;
      read(gout, rENP);

      ac:= 0;

      for j:= 0 to vNivAmbSup.nopartif do  
        begin
          read(gout, r);
          ac:= ac + ABS(r - med^[j]);
        end;

      ac:= 0.5*ac;
      write(gin, ac);

      introENP(gin, totENP, rENP);
    end;

  close(gout);
  close(gin);

  delVector(med, vNivAmbSup.nopartif);
End;
{ ********************************************************** }

{
The levels of data aggregation are referred to as 0, ..,NoMaxNivele, 
where 0 stands for the maximum level of aggregation.

This procedure creates two binary files for a pair given by an item "vNivAmbSup" of
an upper aggregation level "IndAmbSup" and a lower aggregation level "IndAmbInf". 
The 'AUX' and 'REG' files are both named as follows

  'R' + IndAmbSup_IndAmbInf_(Code identifying vNivAmbSup)

They are structured in a common manner. First, they have a common set of initial records in
the file. Second, a sequence of scores are calculated, for each item of the lower level 
of aggregation.

File with extension .aux
========================

* Initial records for the item of the upper aggregation level

 Number of considered items (1) | totPANE (1) | totENP (1) | * (1) | * (1) | 
     IndAmbSup (1) | vNivAmbSup (EqNo+1) | IndAmbInf (1) | noPartiG (1) ||

* A sequence of records for each of those items of the lower aggregation level contained in 
the given item of the upper aggregation level

 vNivAmbInf (EqNo+1) | VR | VRtaD | VRDe | rENP | 0 | ...| noPartiG |

File with extension .reg
========================

* Initial records for the item of the upper aggregation level

 NoItems_considerados (1) | totPANE (1) | totENP (1) | I (1) | Iw (1) | 
     IndAmbSup (1) | vNivAmbSup (EqNo+1) | IndAmbInf (1) | noPartiG (1) ||

* A sequence of records for each of those items of the lower aggregation level contained in 
the given item of the upper aggregation level:

 vNivAmbInf (EqNo+1) | VR | VRtaD | VRDe | VRD | rENP | D | Cox | Inflacci |


 Inflation measures at the item of the lower aggregation level

     rENP : Effective number of parties 
        D : Chibber and Kollman index
      Cox : Cox measure
  Inflacci: Moenius and Kasuya's measure

-----------------------------------------
 Los ndices de los niveles de agregacin varan en: 0, ..,NoMaxNivele

   0 = nivel de agregacin mximo (nacional)
   1 = Agregacin por items del nivel 1
   2 = Agregacin por items del nivel 2
   3 = Agregacin por items del nivel 3

Este procedimiento crea dos ficheros de datos para cada par formado por un item del nivel
ambito superior y un nivel inferior. En concreto, dado un item del nivel ambito superior
"IndAmbSup", se crean los ficheros con extensin
'AUX' y 'REG', respectivamente, ambos con el mismo nombre dado por

  'R' + IndAmbSup_IndAmbInf_(Cdigo de vNivAmbSup)

A continuacin, describimos la estructura de los tipos de ficheros
antes mencionados:


Fichero con extensin aux
=========================

Al principio aparecen EqNo+1+8 registros fijos

 NoItems_considerados (1) | totPANE (1) | totENP (1) | I (1) | Iw (1) | 
     IndAmbSup (1) | vNivAmbSup (EqNo+1) | IndAmbInf (1) | noPartiG (1) ||

Los sitios para I e Iw no contienen en este fichero sus verdaderos valores.
A continuacin, para cada item de nivel inferior aparecern un total
de EqNo+1+ 4 + noPartiG+1  registros:

 vNivAmbInf (EqNo+1) | VR | VRtaD | VRPaco | rENP | 0 | ...| noPartiG |


Fichero con extensin reg
=========================

Al principio aparecen EqNo+1+8 registros fijos

 NoItems_considerados (1) | totPANE (1) | totENP (1) |  I (1) | Iw (1) | 
   IndAmbSup (1) | vNivAmbSup (EqNo+1) | IndAmbInf (1) | noPartiG (1) ||

 I e Iw son ahora las medias, no ponderada y ponderada, 
 de ENP sobre los items del nivel inferior.

A continuacin, para cada item de nivel inferior aparecern  un total
de EqNo+1+ 8

 vNivAmbInf (EqNo+1) | VR | VRtaD | VRPaco | VRD | rENP | D | Cox | Inflacci |


 Inflation measures at lower level

  rENP, totENP : Effective number of parties at the natinal level and district level
             D : Chibber and Kollman index
           Cox : Cox measure
       Inflacci: Moenius and Kasuya's measure

En realidad, las medidas de Chibber and Kollman, Cox and Inflacci
son obtenidas como las medias de las aqu definidas como D, Cox e Inflacci. 

 }

procedure pRegiona(var camino: TcaminFi;
                 IndAmbSup, IndAmbInf: byte);

  var
      iAmbSup: TItmsNivel;
      vNivAmbSup: TRegNivel;
      PaG: PVecParty;
      {noPartiG: TRgPartido;}
      totPANE, totENP: Tnumero;

Begin
  iAmbSup:= 0;

  while GetItm(camino, vNivAmbSup, IndAmbSup, iAmbSup) do
    begin
      GetDistrAmbSup(camino, PaG, totPANE, totENP, vNivAmbSup);
       
      if GetItmsAI_in_AS(camino,IndAmbSup,IndAmbInf,vNivAmbSup, PaG,
			                totPANE, totENP)
      then
        Regional3x(camino, IndAmbSup, IndAmbInf, vNivAmbSup, totENP);

      
      Inc(iAmbSup);
    end;
End;


procedure pRegionalismo(var camino: TcaminFi;
                        NoNiveleUt, IndNivelMx: byte);
{
This procedure calculates the regionalism and party linkage indices for any 
pair of levels of aggregation in the range IndNivelMx..NoNiveleUt. The resulting
scores are saved in two binary files: *.reg and .aux.
}

  var iAS, iAI: byte;

Begin
  for iAS:= IndNivelMx to NoNiveleUt do
    for iAI:= Succ(iAS) to NoNiveleUt do pRegiona(camino, iAS, iAI);

End;


END. 
