{ =====================================================
 Copyright (C) 2007 Francisco A. Ocana & Pablo Onate 

 This file is part of IndElec

 IndElec is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 IndElec is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You have received a copy of the GNU General Public License
 in a file called COPYING which you should have received with
 IndElec. If not, see <http://www.gnu.org/licenses/>.
 ===================================================== 
}
unit DIMSIGLA;

{
Routines focused on the management of some string types saved into special
files: the party acronyms and the names of aggregation levels.
- read a file of party acronyms,
- manage the file of the quotient set of acronyms,
- read names of aggregation levels.

All the string are Pascal-like strings, but not C-like strings (end by null)
}

INTERFACE
{$N+,E-,F+,O+}

uses COMUN;

function leeCadenBlank(var fs: text; pcad: PChar; lenCad: byte): boolean;

function leeCadenaLn(var fs: text; pcad: PChar; lenCad: byte): boolean;

function mismaSigla(var fs: text;
                    var sigla1, sigla2: TcadCandi): boolean;

procedure tomaPolariza(var sigla: TcadCandi; var fpola: text;
                       var xt: Tnumero);

procedure tomaNombreNivel(var nomAmb: TnombreFi; IndiNivel: byte);


IMPLEMENTATION


function leeCadenBlank(var fs: text; pcad: PChar; lenCad: byte): boolean;
{
It reads the characters of the current line of the text file "fs" until a blank is found
or the maximun number of characters, "lenCad", is achieved. Such characters are saved in the 
string referred to "pcad".
}
 
  var c: char;
      cont: byte;
      px: PChar;

Begin
  px:= pcad;
  cont:= 0;

  repeat
    read(fs, c);

    if ((c = ' ') or (c = #9)) then break;

    if cont < lenCad  then
      begin
        Inc(px);
        Inc(cont);
        px^:= c;
      end;

  until Eoln(fs);

  readln(fs);
  pcad^:= chr(cont);

  leeCadenBlank:= (cont = 0); { TRUE: non-significant char is read } 
end;


function leeCadenaLn(var fs: text; pcad: PChar; lenCad: byte): boolean;
{
It reads the characters of the current line until the maximum number of characters, "lenCad",
is achieved.  Such characters except blanks and tabulator are saved
in the string referred to "pcad". 

Notice that the blanks and tabulator characters in the text file are ignored.
}


  var c: char;
      cont: byte;
      px: PChar;


Begin
  px:= pcad;
  cont:= 0;

  repeat
    read(fs, c);

    if ((c = ' ') or (c = #9)) then continue;

    if c = #13 then   {end of line character}
      begin
        read(fs, c);  {new line character, for Windows}
        break;
      end;

    if cont < lenCad  then
      begin
        Inc(px);
        Inc(cont);
        px^:= c;
      end;

  until Eof(fs);

  pcad^:= chr(cont);
  leeCadenaLn:= (cont = 0);  {TRUE: non-significant char is read}
end;



function mismaSigla(var fs: text;
                var sigla1, sigla2: TcadCandi): boolean;

{

  Given
    "sigla1", "sigla2" = two acronyms,
    "fs" = a text file of equivalence classes of acronyms (siglaso.txt),

 it answers the following question: 
 Do these acronyms, "sigla1" and "sigla2", belong to the same  equivalence class?

 The file "fs" contains of the quotient set of party acronyms (siglaso.txt)
}

  var siglax: TcadCandi;
      loca1, loca2: boolean;

Begin
  if sigla1 = sigla2 then
    begin
      mismaSigla:= TRUE;
      exit;
    end;

  reset(fs);
  readln(fs);

  loca1:= FALSE;   loca2:= FALSE;

  repeat
    if leeCadenaLn(fs, @siglax, lenCandida)
    then
      begin
        if (loca1 or loca2) then break;

        loca1:= FALSE;   loca2:= FALSE;
      end
    else
      begin
        if not(loca1) then loca1:= sigla1 = siglax;
        if not(loca2) then loca2:= sigla2 = siglax;

        if loca1 then
         if loca2 then
           begin
             mismaSigla:= TRUE;
             exit;
           end;
      end;

  until Eof(fs);

  mismaSigla:= FALSE;
End;



procedure tomaPolariza(var sigla: TcadCandi; var fpola: text;
                       var xt: Tnumero);

{
It returns the ideological score of the party named by "sigla", if available, from
the file fpola (siglapo.txt).
}

  var siglax: TcadCandi;
      pot: Tnumero;


Begin
  reset(fpola);
  readln(fpola);

  readln(fpola, pot);

  repeat
    if leeCadenaLn(fpola, @siglax, lenCandida)
    then
      begin
        if not(Eof(fpola)) then readln(fpola, pot)
      end
    else
      begin
        if sigla = siglax then
          begin
            xt:= pot;
            exit;
          end;
      end;

  until Eof(fpola);

End;


procedure tomaNombreNivel(var nomAmb: TnombreFi;  IndiNivel: byte);
{
Read the name of the IndiNivel--th aggregation level.
}

  var fconfi: text;
      i: byte;
Begin
  assign(fconfi, camIndElec+'indelec.cfg');   reset(fconfi);

  for i:= 0 to IndiNivel do readln(fconfi);

  leeCadenaLn(fconfi, @nomAmb, 8);

  close(fconfi);
End;

END.
