{ =====================================================
 Copyright (C) 2007 Francisco A. Ocana & Pablo Onate 

 This file is part of IndElec

 IndElec is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 IndElec is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You have received a copy of the GNU General Public License
 in a file called COPYING which you should have received with
 IndElec. If not, see <http://www.gnu.org/licenses/>.
 ===================================================== 
}
unit DIMVECTO;
{$F+,O+}

{
Auxiliary routines for creating/removing general array structures
(vectors, matrixes, symmetric matrixes, etc.) in/from the dynamic memory
(RAM memory).
}

INTERFACE


uses COMUN;

type
   TVector = array[0..NoMaxParti] of Tnumero;
   PVector = ^TVector;

   TMatrix = array[0..NoMaxParti] of PVector;
   PMatrix = ^TMatrix;


{ ******************* Routines for dynamic memory allocating ***************** }

procedure DimVectoR(var v : PVectoR; Ubound: word);
{ ----------------------------------------------------------------------
  the vector  "v" [0..Ubound] is created
  ---------------------------------------------------------------------- }

procedure DimPMatrix(var A : PMatrix; Ubound1: word);
{ ----------------------------------------------------------------------
  the pointer vector "A" [0..Ubound1] (almost a matrix) is created.
  ---------------------------------------------------------------------- }

procedure DimMatrix(var A : PMatrix; Ubound1, Ubound2 : word);
{ ----------------------------------------------------------------------
  the matrix "A" [0..Ubound1, 0..Ubound2] is created.
  ---------------------------------------------------------------------- }

procedure DimSimetri(var S: PMatrix; Ubound: word);
{
  the symmetric matrix "S" [0,..Ubound, 0,..,Ubound] is created
}



{ ****************** Routines for removing dynamic memory *************** }

procedure DelVectoR(var v : PVector; Ubound : word);
{ ----------------------------------------------------------------------
  Removes the vector  "v" [0..Ubound]
  ---------------------------------------------------------------------- }


procedure DelPMatrix(var A : PMatrix; Ubound1: word);
{ ----------------------------------------------------------------------
  Removes the pointer vector "A" [0..Ubound1]
  ---------------------------------------------------------------------- }

procedure DelMatrix(var A : PMatrix; Ubound1, Ubound2 : word);
{ ----------------------------------------------------------------------
  Removes the matrix "A" [0..Ubound1, 0..Ubound2]
  ---------------------------------------------------------------------- }

procedure DelSimetri(var S: PMatrix; Ubound: word);
{
  Removes the symmetric matrix  "S" [0,...,Ubound, 0,...,Ubound]
}



IMPLEMENTATION



const maxDim =  NoMaxParti;
      maxFilas = NoMaxParti;


{type Trgdim = TRgPartido;}


procedure DimVectoR(var v : PVectoR; Ubound : word);

  var VectSize : TlongNatural;

Begin
  if Ubound > maxDim
  then
      v:= NIL
  else
    begin
      VectSize := TlongNatural( Succ(Ubound) ) * SizeOf(Tnumero);
                          { the size (bytes) of vector }

      GetMem(v, VectSize);
    end;
End;
{**************************************************************}

procedure DimPMatrix(var A : PMatrix; Ubound1: word);

 var MatSize : TlongNatural;

Begin
  if (Ubound1 > maxFilas)
  then
      A := NIL
  else
    begin
      MatSize := TlongNatural( Succ(Ubound1) ) * SizeOf(PVectoR);

      GetMem(A, MatSize);
    end;
End;
{ ************************************************************** }

procedure DimMatrix(var A : PMatrix; Ubound1, Ubound2 : word);

 var
     N: word;
     PtrSize,
     VectSize: TlongNatural;

Begin
  if (Ubound1 > maxFILAS) or (Ubound2 > maxDIM)
  then
      A := NIL
  else
    begin
      N := Succ(Ubound1);
      PtrSize := SizeOf(PVector);
      VectSize := TlongNatural( Succ(Ubound2) ) * SizeOf(Tnumero) ;

      GetMem(A, N * PtrSize );
      for N := 0 to Ubound1 do GetMem(A^[N], VectSize);
      
    end;
End;
{**************************************************************}

procedure DimSimetri(var S : PMatrix; Ubound : word);

 var N: word;     
     NuSize: TlongNatural;

Begin
  if (Ubound > maxDim)
  then
      S := NIL
  else
    begin
      N := Succ(Ubound);      
      NuSize:= SizeOf(Tnumero);

      GetMem(S, N * SizeOf(PVector));
      for N:= 0 to Ubound do GetMem(S^[N], Succ(N) * NuSize);        
    end;
End;
{**************************************************************}

procedure DelVectoR(var v : PVectoR; Ubound : word);
Begin
  if v <> NIL then
    begin
      FreeMem(v, TlongNatural( Succ(Ubound) ) * SizeOf(Tnumero) );
      v:= NIL;
    end;
End;
{**************************************************************}

procedure DelPMatrix(var A : PMatrix; Ubound1: word);

Begin
  if A <> NIL then
    begin
      FreeMem(A, TlongNatural( Succ(Ubound1) ) * SizeOf(PVector) );
      A := NIL;
    end;
End;
{******************************************************************}

procedure DelMatrix(var A : PMatrix; Ubound1, Ubound2 : word);

  var
    I        : word;
    VectSize : TlongNatural;

Begin
  if A <> NIL then
    begin
      VectSize := TlongNatural( Succ(Ubound2) ) * SizeOf(Tnumero);
      for I := 0 to Ubound1 do FreeMem(A^[I], VectSize);

      FreeMem(A, TlongNatural( Succ(Ubound1) ) * SizeOf(PVector) );
      A := NIL;
    end;
End;
{******************************************************************}

procedure DelSimetri(var S : PMatrix; Ubound : word);

  var I      : word;
      NuSize : TlongNatural;

Begin
  if S <> NIL then
    begin
      NuSize := SizeOf(Tnumero);
      for I := 0 to Ubound do FreeMem(S^[I], Succ(I)*NuSize);

      FreeMem(S, TlongNatural( Succ(Ubound) ) * SizeOf(PVector) );
      S := NIL;
    end;
End;
{******************************************************************}

END.
