{ =====================================================
 Copyright (C) 2007 Francisco A. Ocana & Pablo Onate 

 This file is part of IndElec

 IndElec is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 IndElec is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You have received a copy of the GNU General Public License
 in a file called COPYING which you should have received with
 IndElec. If not, see <http://www.gnu.org/licenses/>.
 ===================================================== 
}
unit VOLADIM;

{
Routines performing the calculation of volatility indexes for each
aggregation level.
}

INTERFACE
{$N+}

uses COMUN, DIMDEFN, DEFNIVEL, VOLDEF,VOLFORMU;

procedure NombraFichVol(var gout: Tfichero; var cami2: TcaminFi;
                        IndNivel: byte);


function pVolatility(var cami1, cami2: TcaminFi;
                     var infoFixx: TOInfoFich;
                     var NoNiveleUt, IndNivelAgrMx: byte): boolean;

IMPLEMENTATION

uses UDOS, DIMSIGLA, DIMAUXN, VOLAGET;


procedure NombraFichVol(var gout: Tfichero; var cami2: TcaminFi;
                        IndNivel: byte);
  var nomv: TnombreFi;
      cad2: string[2];

Begin
  nomv:= 'V';
  str(IndNivel, cad2);  nomv:= nomv + cad2;

  assign(gout, cami2+nomv+'.vx');
End;
{ ****************************************************************** }

procedure iniVol1(var cami: TcaminFi;
                 var Part: PVecParty; var vNivel: TRegNivel;
                 var sumVot, sumEsc: Tnumero);

{
  Dados
     cami = camino al fichero de datos,
     vNivel = nivel de realizacin de clculos,

 toma los resultados al nivel "vNivel" ,  obtiene la distribucin (proporciones)
 de votos y escaos, la ordena de mayor a menor.
 }


Begin
  iniDato2F(cami, Part, vNivel);
  IniNorma(sumVot, sumEsc, Part, vNivel.nopartif);
End;
{ ********************************************************************* }

{
procedure DoLocalizado(Pa: PVecParty;  j, i: TRgPartido;  Pvol: PVecVolati;
                   var sumVot, sumEsc: Tnumero; PRIMER: boolean);

This procedure updates the volatility increments, "Pvol", and the shares of
votes and seat accounted, "sumVot" and "sumEsc", where

 Pa : vector of electoral data of political parties,
 j : index of political party,
 i : index of volatility increment,
 Pvol: vector of volatility increments,
 sumVot: total of votes,
 sumEsc: total of seats,
 PRIMER: Is Pa from the first election?

Begin
  sumVot:= sumVot + Pa^[j].Vp;
  sumEsc:= sumEsc + Pa^[j].Ep;
  Pa^[j].PANE := TRUE;

  if PRIMER
  then
    begin
      Pvol^[i].Vp:= Pvol^[i].Vp - Pa^[j].Vp;
      Pvol^[i].Ep:= Pvol^[i].Ep - Pa^[j].Ep;
    end
  else
    begin
      Pvol^[i].Vp:= Pvol^[i].Vp + Pa^[j].Vp;
      Pvol^[i].Ep:= Pvol^[i].Ep + Pa^[j].Ep;
    end;
End;
}

procedure IniDistribN(var fv: text;  var cami: TcaminFi;
                      Pvol: PVecVolati;   dimVol: TRgPartido;
                      Pa: PVecParty;  noParti: TRgPartido;
                      PRIMER: boolean;
                      var sumVot, sumEsc: Tnumero);

{
Given
      fv: an assigned text file implementing the volatility increments of "parties",
      Pa: distribution of electoral data for a given election,
 noParti: number of data (parties),
  PRIMER: Is the first election?
    Pvol: a vector of volatility increments
  dimVol: number of increments,
    cami: data directory,

this procedure initializes the vector of volatility increments, Pvol, from the data of
an election and "sumVot" and "sumEsc" that account the total of votes and seats
considered in such increments.

   fv = fichero asignado con la informacin relativa a la relacin
      entre siglas de partidos en primer y segundo perodo, es decir,
      contiene la definicin de los incrementos.

PRIMER = estable el perodo de los datos a incluir (PRIMER PERIODO,
       si PRIMER = true y SEGUNDO PERIODO, si PRIMER = false),

    Pa = distribucin de votos y escaoos del perodo a considerar.
noParti = nmero de partidos (-1) en "Pa".

  cami = camino para acceder a los datos y a otros asociados del
       perodo considerado.

  Pvol = vector de incrementos definidos para el clculo de la volatilidad.

  dimVol = nmero de incrementos (-1).

 Este procedimiento realiza el clculo de los incrementos, mediante la
inclusin inteligente de cada una de las distribuciones de partidos
considerada. Al final, el vector "Pa" est vaco. Asimiso, devuelve
en "sumVot" y "sumEsc" los porcentajes de votos y escaos de los partidos
considerados para construir los incrementos.
}

  label encontrado1, encontrado2;

  var fcla: text;
      localiza: boolean;
      saltoLinea, topeSalto: byte;
      siglaN: TcadCandi;
      i, j: TRgPartido;

 procedure DoLocalizado;
{
This is a local procedure in IniDistribN, which updates the volatility increments, 
"Pvol", and the shares of votes and seats accounted, "sumVot" and "sumEsc", where

 Pa : vector of electoral data of political parties,
 j : index of political party,
 i : index of volatility increment,
 Pvol: vector of volatility increments,
 sumVot: total of votes,
 sumEsc: total of seats,
 PRIMER: Is Pa from the first election?
}
 begin
   sumVot:= sumVot + Pa^[j].Vp;
   sumEsc:= sumEsc + Pa^[j].Ep;
   Pa^[j].PANE := TRUE;

   if PRIMER
   then
     begin
       Pvol^[i].Vp:= Pvol^[i].Vp - Pa^[j].Vp;
       Pvol^[i].Ep:= Pvol^[i].Ep - Pa^[j].Ep;
     end
   else
     begin
       Pvol^[i].Vp:= Pvol^[i].Vp + Pa^[j].Vp;
       Pvol^[i].Ep:= Pvol^[i].Ep + Pa^[j].Ep;
     end;
 end; { of the local procedure DoLocalizado }


Begin
  if PRIMER then topeSalto:= 1     { first election  }
            else topeSalto:= 0;    { second election }

  sumVot:= 0;      sumEsc:= 0;

  for j:=0 to noParti do Pa^[j].PANE := FALSE;
{ 
 The field PANE identifies whether the party data has been included 
in the volatility increments:
  PANE = TRUE   = included in an increment
  PANE = FALSE  = not (yet) included in an increment 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 1st searching volatility increments in the file  (a quick search)
}

  for j:= 0 to noParti do
   begin { 1 }
     reset(fv);

     for i:= 0 to 4 do readln(fv); {skip the first 5 lines }

     localiza:= FALSE;

     for i:= 0 to dimVol do
      begin { 2 }
        readln(fv);  {skip bloc char }
        saltoLinea:= 0;

        repeat
          if leeCadenaLn(fv, @siglaN, lenCandida)
          then
            Inc(saltoLinea)
          else
            if (saltoLinea = topeSalto) then
             begin { 4 }
               localiza:= siglaN = Pa^[j].SiglasCandi;

               if localiza then
                begin { 5 }
                  DoLocalizado; {(Pa, j, i, Pvol, sumVot, sumEsc, PRIMER);}
                  goto encontrado1;
                end; { 5 }
             end; { 4 }

        until (saltoLinea = 2) or Eof(fv);
      end; { 2 }

     encontrado1:
   end; { 1 }
 
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
2nd searching volatility increments in the file fv and
the file of equivalence classes (more complex).
}

  assign(fcla, cami+'siglaso.txt');   reset(fcla);

  for j:= 0 to noParti do
   begin { 1 }
     if Pa^[j].PANE then continue;

     reset(fv);

     for i:= 0 to 4 do readln(fv);

     localiza:= FALSE;

     for i:= 0 to dimVol do
      begin { 2 }
        readln(fv);
        saltoLinea:= 0;

        repeat
          if leeCadenaLn(fv, @siglaN, lenCandida)
          then
            Inc(saltoLinea)
          else
            if (saltoLinea = topeSalto) then
             begin { 4 }
               localiza:= mismaSigla(fcla, Pa^[j].SiglasCandi, siglaN);

               if localiza then
                begin { 5 }
                 DoLocalizado; {(Pa, j, i, Pvol, sumVot, sumEsc, PRIMER);}
                 goto encontrado2;
                end; { 5 }
             end; { 4 }

        until (saltoLinea = 2) or Eof(fv);
      end; { 2 }

     encontrado2:
   end; { 1 }

  close(fcla);  
  {FreeMem(Pa, TlongNatural( sizeOf(Tparty) )* Succ(noParti) ); atencin argumento}
End;
{ *************************************************************** }


procedure MeteIncrem(var gout: Tfichero; Pvol: PVecVolati;
                     dimVol: TRgPartido);
  var i: TRgPartido;
Begin
  for i:= 0 to dimVol do write(gout, Pvol^[i].Vp, Pvol^[i].Ep);
End;
{ *************************************************************** }

procedure resultadov(Pvol: PVecVolati; dimVol: TRgPartido;
                    var vSimbo: TSimbAfili;  NoBloqVol: byte;
                    VOTOS: boolean; var gout: Tfichero;
                    var sumPer2, sumPer1: Tnumero);

  var TV, BV, WV: Tnumero;

Begin
  TV:= fTV(Pvol, dimVol, VOTOS);
  aprVolBM(TV,sumPer2,sumPer1); {correction}
  write(gout, TV);

  BV:= fBV(Pvol, dimVol, vSimbo, NoBloqVol, VOTOS);
  aprVolBM(BV,sumPer2,sumPer1); {correction}
  write(gout, BV);

  WV:= TV - BV;
  write(gout, WV);

  if TV > 0
  then
    begin
      BV:= 100*(BV/TV);
      WV:= 100*(WV/TV);
    end
  else
    begin
      BV:= 0;
      WV:= 0;
    end;

  write(gout, BV, WV);
End;
{ **************************************************************** }
{
function pVolati(var cami1, cami2: TcaminFi;
                 var infoFixx: TOInfoFich;
                 var vNivel1, vNivel2: TRegNivel;
                 var gout: Tfichero): boolean;

  var Part1, Part2: PVecParty;
      noPart1, noPart2, dimVol: TRgPartido;
      sumVot2, sumEsc2, sumVot1, sumEsc1: Tnumero;
      fv: text;
      Pvol: PVecVolati;
      vSimbo: TSimbAfili;
      NoBloq: byte;

Begin
  iniVol1(cami1, Part1, noPart1, vNivel1, sumVot1, sumEsc1);

  iniVol1(cami2, Part2, noPart2, vNivel2, sumVot2, sumEsc2);

  WriteNivel(gout, vNivel2);

  iniVolatili(Pvol, dimVol);

  assign(fv, infoFixx.camin+infoFixx.nom+'.ivo');

  IniDistribN(fv, cami2, Pvol, dimVol, Part2, noPart2, FALSE,
              sumVot2, sumEsc2);

  IniDistribN(fv, cami1, Pvol, dimVol, Part1, noPart1, TRUE,
              sumVot1, sumEsc1); modificado

  close(fv);

  write(gout, sumVot2, sumEsc2, sumVot1, sumEsc1);

  IniSimbolo(vSimbo, NoBloq);

  Resultadov(Pvol, dimVol, vSimbo, NoBloq, TRUE, gout,sumVot2,sumVot1);
  Resultadov(Pvol, dimVol, vSimbo, NoBloq, FALSE, gout,sumEsc2,sumEsc1);

  MeteIncrem(gout, Pvol, dimVol);

  FreeMem(Pvol, sizeOf(TVolati) * word( Succ(dimVol) ) );
  pVolati:= TRUE;
End;
}

function pVolati(var cami1, cami2: TcaminFi;
                 var infoFixx: TOInfoFich;
                 var vNivel1, vNivel2: TRegNivel;
                 var gout: Tfichero): boolean;

  var Part: PVecParty;
      dimVol: TRgPartido;
      sumVot2, sumEsc2, sumVot1, sumEsc1: Tnumero;
      fv: text;
      Pvol: PVecVolati;
      vSimbo: TSimbAfili;
      NoBloq: byte;

Begin
  WriteNivel(gout, vNivel2);

{ 2nd election }
  iniVol1(cami2, Part, vNivel2, sumVot2, sumEsc2);

  iniVolatili(Pvol, dimVol);
  assign(fv, infoFixx.camin+infoFixx.nom+'.ivo');

  IniDistribN(fv, cami2, Pvol, dimVol, Part, vNivel2.nopartif, FALSE,
              sumVot2, sumEsc2);
  delVectParty(Part, vNivel2.nopartif);

{ 1st election }
  iniVol1(cami1, Part, vNivel1, sumVot1, sumEsc1);

  IniDistribN(fv, cami1, Pvol, dimVol, Part, vNivel1.nopartif, TRUE,
              sumVot1, sumEsc1);
  delVectParty(Part, vNivel1.nopartif);

  close(fv);

  write(gout, sumVot2, sumEsc2, sumVot1, sumEsc1);

  IniSimbolo(vSimbo, NoBloq);

  Resultadov(Pvol, dimVol, vSimbo, NoBloq, TRUE, gout,sumVot2,sumVot1);
  Resultadov(Pvol, dimVol, vSimbo, NoBloq, FALSE, gout,sumEsc2,sumEsc1);

  MeteIncrem(gout, Pvol, dimVol);

  FreeMem(Pvol, sizeOf(TVolati) * TlongNatural( Succ(dimVol) ) );
  pVolati:= TRUE;
End;


procedure MeteCgd(var gout: Tfichero);
  var f: text;
      r: Tnumero;
      Cgx: word;
      dimVol: TRgPartido;
Begin
  assign(f, camIndElec+'ALafili.vox');   reset(f);

  readln(f, dimVol);
  r:= dimVol;  write(gout, r);

  readln(f, Cgx);
  r:= Cgx;   write(gout, r);

  readln(f, Cgx);
  r:= Cgx;   write(gout, r);
  close(f);
End;


procedure volatixNivel(var cami1, cami2: TcaminFi;
                       var infoFixx: TOInfoFich;
                       IndNivel: byte);
{
  Crea asociado al nivel de agregacin "IndNivel" un fichero binario denominado
  v+"IndNivel".vx que contiene los ndices de volatilidad asociados
  a cada uno de los items de dicho nivel de agregacin.

  Estructura de v+"IndNivel".vx
  =============================

  (*) Registros iniciales: (5)

  No_de_Items (1) | IndNivel (1) | dimVol | Cg2 (1) | Cg1 (1) |

  (*) Para cada item aparece

  vNivel2 (EqNo+1) | 
  SumVot_2 (1) | SumEsc_2 (1) | SumVot_1 (1) | SumEsc_1 (1) |
       TV_e | BV_e | WBV_e | %BV_e | %WBV_e |
       TV_p | BV_p | WBV_p | %BV_p | %WBV_p |

  Pvol[0].V | Pvol[0].E | ...| Pvol[dimVol].V | Pvol[dimVol].E |

}

  label otror;

  var iv2, iv1, conta: TItmsNivel;
      gout: Tfichero;
      vNiv2, vNiv1: TRegNivel;
      r: Tnumero;
      NoIgualNivel: boolean;

Begin
  iv2:= 0;  

  NombraFichVol(gout, cami2, IndNivel);
  rewrite(gout);

  r:= 0;         write(gout, r);

  conta:= 0;

  r:= IndNivel;  write(gout, r);

  MeteCgd(gout);

  while GetItm(cami2, vNiv2, IndNivel, iv2) do
    begin
      iv1:= 0; NoIgualNivel:= TRUE;
      
      while GetItm(cami1, vNiv1, IndNivel, iv1) do
        begin
          if vNiv1.CgClasifica[IndNivel] = vNiv2.CgClasifica[IndNivel] then
            begin
              NoIgualNivel:= FALSE;
              break;
            end;

          inc(iv1);
        end;
          
      if NoIgualNivel then goto otror;       

      if pVolati(cami1, cami2, infoFixx, vNiv1, vNiv2, gout)
      then
          Inc(conta);

      otror:
      Inc(iv2);
    end;

  if conta = 0
  then
    begin
      close(gout);

      NombraFichVol(gout, cami2, IndNivel);
      erase(gout);
    end
  else
    begin
      seek(gout, 0);
      r:= Pred(conta);
      write(gout, r);
      close(gout);
    end;

End;


function pVolatility(var cami1, cami2: TcaminFi;
                     var infoFixx: TOInfoFich;
                     var NoNiveleUt, IndNivelAgrMx: byte): boolean;

{
Dados
   "camixx" = camino de fichero
    "nomxx" = nombre de fichero

obtiene los ndices de volatilidad a partir de la informacin considerada
en el fichero "camixx"+"nomx"+.IVO .
}

  var
      IndNivel: byte;

Begin
  if InicVolati(cami1, cami2, infoFixx) then
    begin
      pVolatility:= TRUE;
      exit;
    end;

  if CheckNivelUtif(NoNiveleUt, IndNivelAgrMx, cami1, cami2) then
   begin
      pVolatility:= TRUE;
      exit;
    end;

  Remueve(cami2, 'vx');

  for IndNivel:= IndNivelAgrMx to NoNiveleUt do
       volatixNivel(cami1, cami2, infoFixx, IndNivel);


  pVolatility:= FALSE;
End;

END.
