{ =====================================================
 Copyright (C) 2007 Francisco A. Ocana & Pablo Onate 

 This file is part of IndElec

 IndElec is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 IndElec is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You have received a copy of the GNU General Public License
 in a file called COPYING which you should have received with
 IndElec. If not, see <http://www.gnu.org/licenses/>.
 ===================================================== 
}
unit VOLAGET;

{
Routines for managing the information on increments experienced by
parties between two consecutive elections, 
when volatility indexes are calculated.
}

INTERFACE

uses COMUN;

function InicVolati(var cami1, cami2: TcaminFi;
                    var infoFixx: TOInfoFich): boolean;

function CheckNivelUtif(var NoNiveleUt, IndNivelAgrMx: byte;
                        var camino1, camino2: TcaminFi): boolean;


IMPLEMENTATION

uses COMUTIL, DIMSIGLA, DIMAUXN, VOLDEF, Dialogs;


function CheckCharBloq(ca: char; var vSimbo: TSimbAfili;
                       NoBloqVol: byte): boolean;

  var l: byte;
Begin
  for l:= 0 to NoBloqVol do
    if vSimbo[l] = ca then
      begin
        CheckCharBloq:= FALSE;
        exit;
      end;

  CheckCharBloq:= TRUE;
End;


function InicVolati(var cami1, cami2: TcaminFi;
                    var infoFixx: TOInfoFich) : boolean;

{
  Given 
    infoFixx : file defining the volatility records (*.ivo)

this function returns the two working paths, "cami1" and "cami2" for each of the two periods,
counts the volatility records and reads the labels of the blocs of parties, 
which are saved into the text file camIndElec+'ALafili.vox'. 
It returns TRUE when some error is detected; FALSE, otherwise.
}

  var fv, fout: text;
      siglax: TcadCandi;
      saltoLinea: byte;
      i, dimVol: TRgPartido;
      Cg1, Cg2: string[8];
      vSimbo: TSimbAfili;
      NoBloqVol: byte;


Begin
  {$I-}
  assign(fv, infoFixx.camin + infoFixx.nom+'.ivo');  reset(fv);
  {$I+}

  if IoResult <> 0 then
    begin
      inicVolati:= TRUE;
      exit;
    end;

  readln(fv, cami2);
  leeCadenaLn(fv, @Cg2, 8);

  readln(fv, cami1);
  leeCadenaLn(fv, @Cg1, 8);

  CheckPath(cami2);
  CheckPath(cami1);

  readln(fv);

  dimVol:= 0;
  saltoLinea:= 0;

  repeat
    if leeCadenaLn(fv, @siglax, lenCandida) then Inc(saltoLinea);

    if (saltoLinea = 2) then
      begin
        Inc(dimVol);
        saltoLinea:= 0;
      end;

  until Eof(fv);

  if saltoLinea = 0 then Dec(dimVol);

  assign(fout, camIndElec+'ALafili.vox');   rewrite(fout);

  writeln(fout, dimVol);
  writeln(fout, Cg2);
  writeln(fout, Cg1);

  reset(fv);
  for i:= 0 to 4 do readln(fv);

  iniSimbolo(vSimbo, NoBloqVol);

  for i:= 0 to dimVol do
    begin
      leeCadenaLn(fv, @siglax, lenCandida);
      siglax[1]:= upcase(siglax[1]);
      writeln(fout, siglax[1]);

      if CheckCharBloq(siglax[1], vSimbo, NoBloqVol) then
         begin
           ShowMessage('Found a non-defined bloc label');
           close(fv);
           close(fout);
           inicVolati:= TRUE;
           exit;
         end;

      saltoLinea:= 0;

      repeat
        if leeCadenaLn(fv, @siglax, lenCandida) then Inc(saltoLinea);
      until (saltoLinea = 2)  or Eof(fv);
    end;

  close(fout);
  close(fv);

  inicVolati:= FALSE;
End;

function CheckNivelUtif(var NoNiveleUt, IndNivelAgrMx: byte;
                        var camino1, camino2: TcaminFi): boolean;

 var IndNivelAgrMx1, NoNiveleUt1: byte;
Begin
  NuNiveleUtif(NoNiveleUt, IndNivelAgrMx, camino2);
  NuNiveleUtif(NoNiveleUt1, IndNivelAgrMx1, camino1);


  if (NoNiveleUt <> NoNiveleUt1) then
    begin
      CheckNivelUtif:= TRUE;
      exit;    
    end;


  if (IndNivelAgrMx <> IndNivelAgrMx1) then
    begin
      CheckNivelUtif:= TRUE;
      exit;
    end;

  CheckNivelUtif:= FALSE;
End;

END.
