{ =====================================================
 Copyright (C) 2007 Francisco A. Ocana & Pablo Onate 

 This file is part of IndElec

 IndElec is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 IndElec is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You have received a copy of the GNU General Public License
 in a file called COPYING which you should have received with
 IndElec. If not, see <http://www.gnu.org/licenses/>.
 ===================================================== 
}
unit XDIMEN;

{
Routines implementing the stages of the step-by-step run process
of the module Dimensi with aggregation levels. This unit	
connects module Dimensi with the IndElec GUI.
}

INTERFACE

uses COMUN, DEFNIVEL, DIMAUXN, CONSDATO,
     UNIVELES, DIMEN, Dialogs;

procedure xdimen1w(var infoFi: TOInfoFich);
{ 
 Lee el fichero infoFi.camin+infoFi.nom.dab por primera vez y crea la base de
 datos en el fichero infoFi.camin+'partido.bda' . En dicha base de datos se habrn
 almacenado para cada item, correspondiente a cada nivel de agregacin considerado, 
 las candidaturas, los votos y los escaos obtenidos.      
}

procedure xdimen2w(var infoFi: TOInfoFich);
{
  Esta funcin extrae del fichero "Partido.bda" todas las siglas de los distintos 
  partidos y, posteriormente, las almacena en el fichero "siglas.txt" en orden 
  creciente, asignando el caracter PANE o NO-PANE.
}

procedure xdimen3w(var infoFi: TOInfoFich);

procedure xdimen4w(var infoFi: TOInfoFich);

procedure xdimen5w(var infoFi: TOInfoFich);

procedure xdimen6w(var infoFi: TOInfoFich);


IMPLEMENTATION


procedure xdimen1w(var infoFi: TOInfoFich);
Begin
  if lecturaTXT(infoFi)
  then
     ShowMessage('Error in the initial data file')
  else
     ShowMessage('Initial database done');
End;
{ ********************************************************** }

procedure xdimen2w(var infoFi: TOInfoFich);

begin
  if codato2(infoFi.camin)
  then
      ShowMessage('Error creating the acronym file template')
  else
      ShowMessage('File of acronyms done');

end;

{ ********************************************************** }

procedure xdimen3w(var infoFi: TOInfoFich);
{
  Esta funcin lleva a cabo la lectura del fichero "siglas.txt" y, ms concretamente, las
  asignaciones de partidos polticos dentro de las clases PANEs y NO-PANEs. Una vez leida
  dichas asignaciones, las incorpora a la base de datos del fichero "partido.bda"
}
begin
  if codato3(infoFi.camin)
  then
      ShowMessage('Error reading siglas.txt')
  else
      ShowMessage('Regionalist party information learned');
end;


{ ********************************************************** }

procedure xdimen4w(var infoFi: TOInfoFich);
{
  Elimina ficheros de clculos intermedios en el camino de trabajo. 
}

begin
  RemuFichCalc(infoFi.camin);

  ShowMessage('Arrangements done');
end;


{ ********************************************************** }

procedure xdimen5w(var infoFi: TOInfoFich);
{
Crea el fichero camino+"cgNivelo.aux" con los cdigos de los items para los que se han 
suministrado datos electorales, los cules sern obtenidos del fichero camino+"partido.bda".

Asimismo, se crean los ficheros .cg correspondientes a cada uno de los niveles 
de agregacin que constituyen la estructura de niveles de agregacin 
especificada en la cabecera del fichero .dab. En concreto, a partir de la estructura
de niveles de agregacin definida en la configuracin de IndElec, se extrae la 
correspondiente estructura de niveles de agregacin especificada en la cabecera
del fichero dab (lineas 2 y 3).

Finalmente, crea una versin preliminar del fichero de clases de equivalencia "siglaso.txt".
}

label finalizando;

var
    NoNiveleUti, IndNivelMx: byte;
    Mnivel: TMatrNivel;

begin
  if RepasoCg(infoFi.camin, NoNiveleUti, IndNivelMx, Mnivel) then
     begin
       ShowMessage('Error in bda: codes of items of aggregation levels');
       exit;
     end;

  { En RepasoCg se inicializan "NoNiveleUti", "IndNivelMx" y "Mnivel" }

  if IniciarNiveles(infoFi.camin, NoNiveleUti, IndNivelMx, Mnivel) then
    begin
      ShowMessage('Error making files of item codes');
      goto finalizando;
    end;

  if codato4(infoFi.camin, Mnivel) then
    begin
      ShowMessage('Error: acronym file (siglas.txt) not found');
      goto finalizando;
    end;

  ShowMessage('Calculations done');

  finalizando:
  finCg(Mnivel);
end;


{ ********************************************************** }

procedure xdimen6w(var infoFi: TOInfoFich);
begin
  pDimeDesprxNiv(infoFi.camin);

  ShowMessage('Calculations done');
end;



END.
